
import unittest
from unittest import mock

from copy import deepcopy
from datetime import datetime, timedelta
from .routing_rules import Weekdays


class WeekdaysTestCase(unittest.TestCase):

    def test_is_valid_schedule(self):
        self.assertTrue(Weekdays.is_valid_schedule(['MON', 'TUE']))
        self.assertFalse(Weekdays.is_valid_schedule(['MON', 'SRC']))

    def test_is_scheduled(self):
        now = datetime.now()
        day = now.strftime('%A')[:3]

        self.assertFalse(Weekdays.is_scheduled_day(now, []))
        self.assertTrue(Weekdays.is_scheduled_day(now, [day]))
        self.assertFalse(Weekdays.is_scheduled_day(now + timedelta(days=1), [day]))

    def test_weekday_dayname(self):
        now = datetime.now()
        day = now.strftime('%A')[:3].upper()
        self.assertEqual(day, Weekdays.dayname(now))


class RoutingRuleSchemeServiceTest(unittest.TestCase):
    """Base class for RoutingRuleSchemeService tests."""

    def setUp(self):
        try:
            from .routing_rules import RoutingRuleSchemeService
        except ImportError:
            # a missing class should result in a failed test, not in an error
            # that prevents all the tests in the module from even being run
            self.fail(
                "Could not import class under test "
                "(RoutingRuleSchemeService).")
        else:
            self.instance = RoutingRuleSchemeService()


@mock.patch(
    'apps.rules.routing_rules'
    '.RoutingRuleSchemeService._validate_routing_scheme')
@mock.patch(
    'apps.rules.routing_rules'
    '.RoutingRuleSchemeService._check_if_rule_name_is_unique')
class OnCreateMethodTestCase(RoutingRuleSchemeServiceTest):
    """Tests for the on_create() method."""

    def test_does_not_modify_semantically_non_empty_schedules(self, *mocks):
        routing_schemes = [{
            'name': 'scheme_1',
            'rules': [{
                'name': 'rule_1',
                'schedule': {
                    'day_of_week': ['MON'],
                    'hour_of_day_from': '08:00:00',
                    'hour_of_day_to': '18:00:00',
                    'time_zone': 'UTC'
                }
            }]
        }]
        original_scheme = deepcopy(routing_schemes[0])
        self.instance.on_create(routing_schemes)
        self.assertEqual(routing_schemes[0], original_scheme)

    def test_does_not_modify_empty_schedules(self, *mocks):
        routing_schemes = [{
            'name': 'scheme_1',
            'rules': [{
                'name': 'rule_1',
                'schedule': None
            }]
        }]
        original_scheme = deepcopy(routing_schemes[0])
        self.instance.on_create(routing_schemes)
        self.assertEqual(routing_schemes[0], original_scheme)

    def test_sets_semantically_empty_schedules_to_none(self, *mocks):
        routing_schemes = [{
            'name': 'scheme_1',
            'rules': [{
                'name': 'rule_1',
                'schedule': {'time_zone': 'UTC'}  # effectively empty schedule
            }]
        }]

        expected_scheme = deepcopy(routing_schemes[0])
        expected_scheme['rules'][0]['schedule'] = None

        self.instance.on_create(routing_schemes)

        self.assertEqual(routing_schemes[0], expected_scheme)


@mock.patch(
    'apps.rules.routing_rules'
    '.RoutingRuleSchemeService._validate_routing_scheme')
@mock.patch(
    'apps.rules.routing_rules'
    '.RoutingRuleSchemeService._check_if_rule_name_is_unique')
class OnUpdateMethodTestCase(RoutingRuleSchemeServiceTest):
    """Tests for the on_update() method."""

    def test_does_not_modify_semantically_non_empty_schedules(self, *mocks):
        routing_scheme = {
            'name': 'scheme_1',
            'rules': [{
                'name': 'rule_1',
                'schedule': {
                    'day_of_week': ['MON'],
                    'hour_of_day_from': '08:00:00',
                    'hour_of_day_to': '18:00:00',
                    'time_zone': 'UTC'
                }
            }]
        }
        original_scheme = deepcopy(routing_scheme)
        self.instance.on_update(routing_scheme, {})
        self.assertEqual(routing_scheme, original_scheme)

    def test_does_not_modify_empty_schedules(self, *mocks):
        routing_scheme = {
            'name': 'scheme_1',
            'rules': [{
                'name': 'rule_1',
                'schedule': None
            }]
        }
        original_scheme = deepcopy(routing_scheme)
        self.instance.on_update(routing_scheme, {})
        self.assertEqual(routing_scheme, original_scheme)

    def test_sets_semantically_empty_schedules_to_none(self, *mocks):
        routing_scheme = {
            'name': 'scheme_1',
            'rules': [{
                'name': 'rule_1',
                'schedule': {'time_zone': 'UTC'}  # effectively empty schedule
            }]
        }

        expected_scheme = deepcopy(routing_scheme)
        expected_scheme['rules'][0]['schedule'] = None

        self.instance.on_update(routing_scheme, {})

        self.assertEqual(routing_scheme, expected_scheme)


class ValidateScheduleMethodTestCase(RoutingRuleSchemeServiceTest):
    """Tests for the _validate_schedule() method."""

    def setUp(self):
        super().setUp()
        self.schedule = {
            'day_of_week': ['WED', 'FRI']
        }

    @mock.patch(
        'apps.rules.routing_rules.all_timezones_set',
        {'Foo/Bar', 'Foo/Baz', 'Here/There'}
    )
    def test_raises_error_on_unknown_time_zone(self):
        from superdesk.errors import SuperdeskApiError

        self.schedule['time_zone'] = 'Invalid/Zone'

        with self.assertRaises(SuperdeskApiError) as context:
            self.instance._validate_schedule(self.schedule)

        # also check the error message
        msg = context.exception.message
        msg = msg.lower() if msg else ''
        self.assertIn('time zone', msg)
        self.assertIn('invalid/zone', msg)

    def test_allows_empty_end_time(self):
        self.schedule['hour_of_day_from'] = '10:30:00'
        self.schedule['hour_of_day_to'] = ''

        try:
            self.instance._validate_schedule(self.schedule)
        except Exception as ex:
            self.fail(
                "Unexpected exception on empty hour_of_day_to: {}".format(ex)
            )


class GetScheduledRoutingRulesMethodTestCase(RoutingRuleSchemeServiceTest):
    """Tests for the _get_scheduled_routing_rules() method."""

    def test_returns_only_the_rules_scheduled_at_current_time(self):
        rules = [{
            'name': 'rule_1',
            'schedule': {
                'day_of_week': ['SAT'],
                'hour_of_day_from': '01:00:00',
                'hour_of_day_to': '23:00:00',
                'time_zone': 'UTC',
            }
        }, {
            'name': 'rule_2',
            'schedule': {
                'day_of_week': ['MON', 'TUE', 'WED'],
                'hour_of_day_from': '21:00:00',
                'hour_of_day_to': '22:00:00',
                'time_zone': 'UTC',
            }
        }, {
            'name': 'rule_3',
            'schedule': {
                'day_of_week': ['TUE'],
                'hour_of_day_from': '22:00:00',
                'hour_of_day_to': '23:00:00',
                'time_zone': 'UTC',
            }
        }]

        now = datetime(2015, 8, 18, 21, 30)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], rules[1])

    def test_takes_rule_time_zone_into_account(self):
        rules = [{
            'name': 'rule_berlin',
            'schedule': {
                'day_of_week': ['TUE'],
                'hour_of_day_from': '14:00:00',
                'hour_of_day_to': '15:00:00',
                'time_zone': 'Europe/Berlin',  # UTC +01:00/+02:00
            }
        }, {
            'name': 'rule_singapore',
            'schedule': {
                'day_of_week': ['TUE'],
                'hour_of_day_from': '22:00:00',
                'hour_of_day_to': '23:00:00',
                'time_zone': 'Asia/Singapore',  # always UTC +08:00
            }
        }]

        now = datetime(2015, 9, 15, 14, 30)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], rules[1])

    def test_assumes_utc_time_zone_if_none_set(self):
        rules = [{
            'name': 'rule_berlin',
            'schedule': {
                'day_of_week': ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN'],
                'hour_of_day_from': '14:50:00',
                'hour_of_day_to': '14:55:00',
                'time_zone': None,
            }
        }]

        now = datetime(2015, 9, 15, 14, 52)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], rules[0])

    def test_assumes_from_the_start_of_the_day_if_start_time_not_set(self):
        rules = [{
            'name': 'rule_berlin',
            'schedule': {
                'day_of_week': ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN'],
                'hour_of_day_from': None,
                'hour_of_day_to': '20:30:00',
                'time_zone': 'UTC',
            }
        }]
        now = datetime(2015, 9, 15, 0, 0, 0, 0)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(result, rules)

    def test_assumes_until_the_end_of_the_day_if_end_time_not_set(self):
        rules = [{
            'name': 'rule_berlin',
            'schedule': {
                'day_of_week': ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN'],
                'hour_of_day_from': '18:45:00',
                'hour_of_day_to': None,
                'time_zone': 'UTC',
            }
        }]
        now = datetime(2015, 9, 15, 23, 59, 59, 999999)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(result, rules)

    def test_schedule_routing_rules_at_end_time(self):
        rules = [{
            'name': 'rule_berlin',
            'schedule': {
                'day_of_week': ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN'],
                'hour_of_day_from': '00:00:00',
                'hour_of_day_to': '23:55:00',
                'time_zone': 'UTC',
            }
        }]
        # before end time
        now = datetime(2015, 9, 15, 23, 54, 59, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, rules)

        # one second after end time
        now = datetime(2015, 9, 15, 23, 55, 1, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, rules)

        # after end time
        now = datetime(2015, 9, 15, 23, 55, 59, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, rules)

        # one second after end time
        now = datetime(2015, 9, 15, 23, 56, 0, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, [])

        # one second after end time
        now = datetime(2015, 9, 15, 23, 56, 1)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, [])

        # after end time
        now = datetime(2015, 9, 15, 23, 59, 59, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, [])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource

CONTACTS_PRIVILEDGE = 'contacts'
VIEW_CONTACTS = 'view_contacts'


class ContactsResource(Resource):
    """Resource class for contact items

    A contact can either be for an individual or for an organisation

    """

    schema = {
        # flag to mark the contact item active/inactive for example if the item relates to a contact that is no longer
        # valid
        'is_active': {
            'type': 'boolean',
            'default': True
        },
        # flag to indicate that the contact item should not be publicly visible, for example if the item relates to
        # a contact that can not be distributed due to privacy concerns
        'public': {
            'type': 'boolean',
            'default': True
        },
        'organisation': {
            'type': 'string',
            'required': False
        },
        'first_name': {
            'type': 'string',
            'required': False
        },
        'last_name': {
            'type': 'string',
            'required': False
        },
        'honorific': {
            'type': 'string',
            'required': False
        },
        'job_title': {
            'type': 'string',
            'required': False
        },
        'mobile': {
            'type': 'list',
            'schema': {
                'type': 'dict',
                'schema': {
                    'number': {'type': 'string'},
                    # usage may provide information as to the usage of the number e.g. Business hours only
                    'usage': {'type': 'string'},
                    # If public is true then the number can be made public, if false it is only visible in the system
                    'public': {'type': 'boolean'}
                }
            }
        },
        'contact_phone': {
            'type': 'list',
            'schema': {
                'type': 'dict',
                'schema': {
                    'number': {'type': 'string'},
                    'usage': {'type': 'string'},
                    # If public is true then the number can be made public, if false it is only visible in the system
                    'public': {'type': 'boolean'}
                }
            }
        },
        'fax': {
            'type': 'string',
            'required': False
        },
        'contact_email': {
            'type': 'list',
            'required': False,
            'schema': {
                'type': 'email'
            }
        },
        'twitter': {
            'type': 'string',
            'required': False,
            'twitter': True
        },
        'facebook': {
            'type': 'string',
            'required': False
        },
        'instagram': {
            'type': 'string',
            'required': False
        },
        'website': {
            'type': 'string',
            'required': False
        },
        'contact_address': {
            'type': 'list',
            'schema': {
                'type': 'string',
                'required': False
            }
        },
        'locality': {
            'type': 'string',
            'required': False
        },
        'city': {
            'type': 'string',
            'required': False
        },
        'contact_state': {
            'type': 'string',
            'required': False
        },
        'postcode': {
            'type': 'string',
            'required': False
        },
        'country': {
            'type': 'string',
            'required': False
        },
        'notes': {
            'type': 'string',
            'required': False
        }
    }

    datasource = {
        'source': 'contacts',
        'search_backend': 'elastic'
    }
    item_url = r'regex("[\w]+")'
    privileges = {'POST': CONTACTS_PRIVILEDGE,
                  'PATCH': CONTACTS_PRIVILEDGE,
                  'DELETE': CONTACTS_PRIVILEDGE}


class OrganisationSearchResource(Resource):
    """Search for organisation within the contacts collection

    """

    datasource = {
        'source': 'contacts',
        'search_backend': 'elastic'
    }

    resource_methods = ['GET']
    item_methods = ['GET']
    url = 'contacts/organisations'
    resource_title = 'contacts_organisations'




import pytz
from datetime import datetime, timedelta
from superdesk.tests import set_placeholder
from steps import when, then, get_json_data, parse_date  # @UnresolvedImport
from superdesk.utc import utcnow


@when('we run create content task')
def when_we_run_create_content_task(context):
    from apps.templates import create_scheduled_content
    now = utcnow() + timedelta(days=8)
    with context.app.app_context():
        items = create_scheduled_content(now)
        for item in items:
            set_placeholder(context, 'ITEM_ID', str(item['_id']))


@then('next run is on monday "{time}"')
def then_next_run_is_on_monday(context, time):
    data = get_json_data(context.response)
    next_run = parse_date(data.get('next_run'))
    fmt = '%H:%M:%S'

    try:
        # assume time is set in given time zone
        tz = pytz.timezone(data['schedule']['time_zone'])
    except KeyError:
        # fallback to utc
        tz = pytz.utc

    parsed = datetime.strptime(time, fmt)
    expected = datetime.now(tz)
    expected += timedelta(((-expected.weekday()) + 7) % 7)
    expected = expected.astimezone(tz)
    expected = expected.replace(hour=parsed.hour, minute=parsed.minute, second=parsed.second, microsecond=0)
    expected_utc = expected.astimezone(pytz.utc)

    assert isinstance(next_run, datetime)
    if tz.zone == 'Australia/Sydney':
        assert next_run.weekday() == 6
    else:
        assert next_run.weekday() == 0
    assert next_run.strftime(fmt) == expected_utc.strftime(fmt), \
        'next run %s is not expected %s' % (next_run.strftime(fmt), expected_utc.strftime(fmt))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
from copy import deepcopy
from eve.utils import config, ParsedRequest
from superdesk import get_resource_service
from superdesk.errors import StopDuplication, InvalidStateTransitionError
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE, PUBLISH_SCHEDULE, SCHEDULE_SETTINGS, \
    PROCESSED_FROM, PUBLISH_STATES
from apps.archive.common import ITEM_OPERATION
from apps.publish.content.common import publish_services
from apps.content_types import apply_schema


def internal_destination_auto_publish(item, **kwargs):
    """Auto publish the item using internal destination

    :param dict item: item to be published
    :param kwargs:
    :raises StopDuplication: to indicate the superdesk.internal_destination.handle_item_published
    to stop duplication as duplication is handle by this method.
    """
    if item.get(ITEM_STATE) not in PUBLISH_STATES:
        raise InvalidStateTransitionError(message='Internal Destination auto publish macro can '
                                                  'only be called after publishing the item.')

    operation = item.get(ITEM_OPERATION)
    archive_action_service = get_resource_service(publish_services.get(operation))
    archive_service = get_resource_service('archive')
    extra_fields = [PUBLISH_SCHEDULE, SCHEDULE_SETTINGS]
    # if any macro is doing publishing then we need the duplicate item that was published earlier
    req = ParsedRequest()
    req.where = json.dumps({
        '$and': [
            {PROCESSED_FROM: item.get(config.ID_FIELD)},
            {'task.desk': str(item.get('task').get('desk'))}
        ]
    })
    req.max_results = 1
    overwrite_item = next((archive_service.get_from_mongo(req=req, lookup=None)), None)

    if item.get(ITEM_STATE) == CONTENT_STATE.PUBLISHED or not overwrite_item:
        new_id = archive_service.duplicate_content(item, state='routed', extra_fields=extra_fields)
        get_resource_service('archive_publish').patch(
            id=new_id,
            updates={ITEM_STATE: item.get(ITEM_STATE), PROCESSED_FROM: item[config.ID_FIELD]}
        )
    else:
        if overwrite_item:
            # get the schema fields
            schema_item = apply_schema(deepcopy(item))
            keys_to_delete = ['source', 'unique_id', 'unique_name', 'original_id',
                              'expiry', 'correction_sequence']
            # remove the keys
            archive_service.remove_after_copy(schema_item, delete_keys=keys_to_delete)
            # get the diff
            updates = {key: val for key, val in schema_item.items()
                       if overwrite_item.get(key) != val and not key.startswith("_")}
            archive_action_service.patch(id=overwrite_item[config.ID_FIELD],
                                         updates=updates)

    # raise stop duplication on successful completion so that
    # internal destination superdesk.internal_destination.handle_item_published
    # will not duplicate the item.
    raise StopDuplication()


name = 'Internal_Destination_Auto_Publish'
label = 'Internal Destination Auto Publish'
callback = internal_destination_auto_publish
access_type = 'backend'
action_type = 'direct'



"""Superdesk places module.

Provides places autocomplete and search.
"""

import superdesk

from .places_autocomplete import PlacesAutocompleteResource, PlacesAutocompleteService


def init_app(app):
    if app.config.get('GEONAMES_USERNAME'):
        superdesk.register_resource('places_autocomplete', PlacesAutocompleteResource, PlacesAutocompleteService,
                                    _app=app)




# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""This module contains tools to help using Elastic Search"""

import logging
import json
import pytz
from datetime import datetime
from superdesk import app

logger = logging.getLogger(__name__)

REPOS = ["ingest", "archive", "published", "archived"]
POST_FILTER_MAP = {
    "type": "type",
    "desk": "task.desk",
    "genre": "genre.name",
    "category": "anpa_category.name",
    "urgency": "urgency",
    "priority": "priority",
    "source": "source",
    "legal": "flags.marked_for_legal",
    "sms": "flags.marked_for_sms",
    "language": "language",
}
SEARCH_CVS = [
    {"id": "subject", "name": "Subject", "field": "subject", "list": "subjectcodes"},
    {"id": "companycodes", "name": "Company Codes", "field": "company_codes", "list": "company_codes"},
]
DATE_FORMAT = "%d/%m/%Y"
DATE_FILTERS = {
    "last_month": {"lte": "now-1M/M", "gte": "now-1M/M"},
    "last_week": {"lte": "now-1w/w", "gte": "now-1w/w"},
    "last_day": {"lte": "now-1d/d", "gte": "now-1d/d"},
    "last_24_hours": {"gte": "now-24H"},
    "last_7_hours": {"gte": "now-8H"},
}

DATE_FIELDS = ("firstcreated", "versioncreated", "firstpublished", "schedule_settings.utc_publish_schedule")


def get_index(repos=None):
    """Get index id for all repos."""
    if repos is None:
        repos = REPOS
    indexes = {app.data.elastic.index}
    for repo in repos:
        indexes.add(app.config["ELASTICSEARCH_INDEXES"].get(repo, app.data.elastic.index))
    return ",".join(indexes)


def get_docs(query_result):
    """Get documents from ES query result

    :param dict query_result: ES query result, as returned by service.search
    :return list: found documents
    """
    try:
        docs = [h["_source"] for h in query_result["hits"]["hits"]]
    except KeyError:
        logger.warning(u"Can't retrieve doc from ES results: {data}".format(data=query_result))
        docs = []
    return docs


def filter2query(filter_, user_id=None):
    """Convert filter syntax to ElasticSearch query

    :param dict filter_: filter syntax, as used in saved_searches endpoint
    :return dict: ElasticSearch query DSL usable with service.search method
    """
    # we'll delete key while we handle them, to check that all has been managed at the end
    search_query = filter_["query"].copy()
    query_must = []
    query_must_not = []
    post_filter = []
    post_filter_must_not = []

    # controlled vocabularies can be overriden in settings
    search_cvs = app.config.get("search_cvs", SEARCH_CVS)

    for cv in search_cvs:
        if cv["id"] in search_query and cv["field"] != cv["id"]:
            terms = json.loads(search_query.pop(cv["id"]))
            query_must.append({"terms": {cv["field"] + ".qcode": terms}})

    try:
        query_string = search_query.pop("q")
    except KeyError:
        pass
    else:
        for cv in search_cvs:
            if cv["field"] != cv["id"]:
                query_string.replace(cv["id"] + ".qcode:(", cv["field"] + "q.code:(")
        query_must.append({"query_string": {"query": query_string, "default_operator": "AND"}})

    to_delete = []
    for key, value in search_query.items():
        if key == "from_desk":
            desk = value.split("-")
            if len(desk) == 2:
                if desk[1] == "authoring":
                    query_must.append({"term": {"task.last_authoring_desk": desk[0]}})
                else:
                    query_must.append({"term": {"task.last_production_desk": desk[0]}})
            else:
                logger.warning('unexpected "from_desk" value: {value}'.format(value=value))
        elif key == "to_desk":
            desk = value.split("-")
            if len(desk) == 2:
                query_must.append({"term": {"task.desk": desk[0]}})
                if "from_desk" not in filter_["query"]:
                    if desk[1] == "authoring":
                        field = "task.last_production_desk"
                    else:
                        field = "task.last_authoring_desk"
                    query_must.append({"exists": {field: field}})
            else:
                logger.warning('unexpected "from_desk" value: {value}'.format(value=value))
        elif key == "spike":
            if value == "include":
                pass
            elif value == "only":
                query_must.append({"term": {"state": "spiked"}})
            else:
                query_must_not.append({"term": {"state": "spiked"}})
        elif key == "featuremedia" and value:
            query_must.append({"exists": {"field": "associations.featuremedia"}})
        elif key == "subject":
            terms = json.loads(value)
            query_must.append(
                {
                    "bool": {
                        "should": [{"terms": {"subject.qcode": terms}}, {"terms": {"subject.parent": terms}}],
                        "minimum_should_match": 1,
                    }
                }
            )
        elif key == "company_codes":
            terms = json.loads(value)
            query_must.append({"terms": {"company_codes.qcode": terms}})
        elif key == "marked_desks":
            terms = json.loads(value)
            query_must.append({"terms": {"marked_desks.desk_id": terms}})
        elif key == "ignoreKilled":
            query_must_not.append({"terms": {"state": ["killed", "recalled"]}})
        elif key == "onlyLastPublished":
            query_must_not.append({"term": {"last_published_version": "false"}})
        elif key == "ignoreScheduled":
            query_must_not.append({"term": {"state": "scheduled"}})
        else:
            continue
        to_delete.append(key)

    for key in to_delete:
        del search_query[key]

    for key, field in POST_FILTER_MAP.items():
        value = search_query.pop(key, None)
        if value is not None:
            try:
                post_filter.append({"terms": {field: json.loads(value)}})
            except TypeError as e:
                logger.error('Invalid data received for post filter "{key}": {e}\ndata: {value}'.format(
                    key=key, e=e, value=value))
                # the value is probably not JSON encoded as expected, we try directly the value
                post_filter.append({"terms": {field: value}})
        else:
            value = search_query.pop("not" + key, None)
            if value is not None:
                post_filter_must_not.append({"terms": {field: json.loads(value)}})

    # used by AAP multimedia datalayer
    credit_qcode = search_query.pop("creditqcode", None)
    if credit_qcode is not None:
        values = json.loads(credit_qcode)
        post_filter.append({"terms": {"credit": [v["value"] for v in values]}})

    # date filters
    tz = pytz.timezone(app.config["DEFAULT_TIMEZONE"])
    range_ = {}
    to_delete = []
    for field in DATE_FIELDS:
        value = search_query.get(field)
        if value in DATE_FILTERS:
            range_[field] = DATE_FILTERS[value]
            to_delete.append(field)
        else:
            field_suff = field + "to"
            value = search_query.get(field_suff)
            if value:
                to_delete.append(field_suff)
                field_range = range_.setdefault(field, {})
                try:
                    date = datetime.strptime(value, DATE_FORMAT)
                except ValueError:
                    # the value doesn't correspond to DATE_FORMAT,
                    # it may be using ES date math syntax
                    field_range["lte"] = value
                else:
                    date = tz.localize(datetime.combine(date, datetime.min.time()))
                    field_range["lte"] = date.isoformat()

            field_suff = field + "from"
            value = search_query.get(field_suff)
            if value:
                to_delete.append(field_suff)
                field_range = range_.setdefault(field, {})
                try:
                    date = datetime.strptime(value, DATE_FORMAT)
                except ValueError:
                    # same as above
                    field_range["gte"] = value
                else:
                    date = tz.localize(datetime.combine(date, datetime.max.time()))
                    field_range["gte"] = date.isoformat()

    if range_:
        post_filter.append({"range": range_})
    for key in to_delete:
        del search_query[key]

    # remove other users drafts
    if user_id is not None:
        query_must.append(
            {
                "bool": {
                    "should": [
                        {"bool": {"must": [{"term": {"state": "draft"}}, {"term": {"original_creator": user_id}}]}},
                        {"bool": {"must_not": {"terms": {"state": ["draft"]}}}},
                    ]
                }
            }
        )

    # this is needed for archived collection
    query_must_not.append({"term": {"package_type": "takes"}})

    query = {"query": {"bool": {"must": query_must, "must_not": query_must_not}}}
    if post_filter or post_filter_must_not:
        query["post_filter"] = {"bool": {"must": post_filter, "must_not": post_filter_must_not}}

    if search_query:
        logger.warning(
            "All query fields have not been used, remaining fields: {search_query}".format(search_query=search_query)
        )

    return query



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import pytz
from datetime import datetime, timedelta
from apps.rules.routing_rules import Weekdays
from apps.templates.content_templates import get_next_run, get_item_from_template, render_content_template
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE
from superdesk.tests import TestCase
from superdesk.utc import utcnow


class TemplatesTestCase(TestCase):

    def setUp(self):
        # now is today at 09:05:03
        self.now = datetime.utcnow().replace(hour=9, minute=5, second=3)
        self.weekdays = [day.name for day in Weekdays]

    def get_delta(self, create_at, weekdays, time_zone=None, now=None, cron_list=None):
        schedule = {
            'day_of_week': weekdays,
            'create_at': create_at,
            'is_active': True,
            'time_zone': time_zone or 'UTC'
        }
        if cron_list:
            schedule['cron_list'] = cron_list
            schedule.pop('create_at', None)

        next_run = get_next_run(schedule, now or self.now)
        return next_run - (now or self.now).replace(second=0)

    def test_inactive_schedule(self):
        self.assertEqual(None,
                         get_next_run({'is_active': False, 'day_of_week': self.weekdays, 'create_at': '09:15:00'}))

    def test_next_run_same_day_later(self):
        delta = self.get_delta('09:08:00', self.weekdays)
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 179)

    def test_next_run_same_day_later_cron_list(self):
        cron_list = ['30 07 * * *', '08 09 * * *']
        delta = self.get_delta('09:08:00', self.weekdays, cron_list=cron_list)
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 179)

    def test_next_run_next_day(self):
        delta = self.get_delta('09:03:00', self.weekdays)
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 3600 * 24 - 121)

    def test_next_run_next_week(self):
        delta = self.get_delta('09:03:00', [self.now.strftime('%a').upper()])
        self.assertEqual(delta.days, 6)

    def test_next_run_now(self):
        delta = self.get_delta('09:05:00', self.weekdays)
        self.assertEqual(delta.seconds, 24 * 60 * 60 - 1)

    def test_get_item_from_template(self):
        template = {'_id': 'foo', 'name': 'test',
                    'schedule_desk': 'sports', 'schedule_stage': 'schedule',
                    'data': {
                        'headline': 'Foo',
                        'dateline': {
                            'located': {
                                'city': 'Sydney',
                                'city_code': 'Sydney',
                                'tz': 'Australia/Sydney'
                            },
                            'date': '2015-10-10T10:10:10',
                        }
                    }}
        now = utcnow()
        with self.app.app_context():
            item = get_item_from_template(template)
        self.assertNotIn('_id', item)
        self.assertEqual('foo', item.get('template'))
        self.assertEqual('Foo', item.get('headline'))
        self.assertEqual(CONTENT_STATE.SUBMITTED, item.get(ITEM_STATE))
        self.assertEqual({'desk': 'sports', 'stage': 'schedule'}, item.get('task'))
        dateline = item.get('dateline')
        self.assertEqual('Sydney', dateline['located']['city'])
        self.assertEqual(now, dateline.get('date'))
        self.assertIn('SYDNEY', dateline.get('text'))

    def test_next_run_for_timezone(self):
        # UTC time Zero hours
        now = datetime(2018, 6, 30, 19, 0, 0, 0, tzinfo=pytz.utc)
        current_now = now + timedelta(seconds=5)
        # schedule at 06:00 AM
        delta = self.get_delta('06:00:00',
                               self.weekdays,
                               time_zone='Australia/Sydney',
                               now=current_now
                               )
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 3600)

        # 30 minutes before schedule
        current_now = now + timedelta(minutes=30)
        delta = self.get_delta('06:00:00',
                               self.weekdays,
                               time_zone='Australia/Sydney',
                               now=current_now
                               )
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 1800)

        # hour after schedule
        current_now = now + timedelta(hours=1, seconds=5)
        delta = self.get_delta('06:00:00',
                               self.weekdays,
                               time_zone='Australia/Sydney',
                               now=current_now
                               )
        self.assertEqual(delta.days, 1)


class RenderTemplateTestCase(TestCase):

    def test_render_content_template(self):
        template = {
            '_id': 'foo',
            'template_name': 'test',
            'template_desks': ['sports'],
            'data': {
                'headline': 'Foo Template: {{item.headline}}',
                'body_html': 'This article has slugline: {{item.slugline}} and dateline: {{item.dateline["text"]}} '
                             'at {{item.versioncreated | format_datetime("Australia/Sydney", "%d %b %Y %H:%S %Z")}}',
                'urgency': 1, 'priority': 3,
                'dateline': {},
                'anpa_take_key': 'this is test',
                'place': ['Australia']
            }
        }

        item = {
            '_id': '123', 'headline': 'Test Template',
            'slugline': 'Testing', 'body_html': 'This is test story',
            'dateline': {
                'text': 'hello world'
            },
            'urgency': 4, 'priority': 6,
            'versioncreated': '2015-06-01T22:54:53+0000',
            'place': ['NSW']
        }

        updates = render_content_template(item, template)
        self.assertEqual(updates['headline'], 'Foo Template: Test Template')
        self.assertEqual(updates['urgency'], 1)
        self.assertEqual(updates['priority'], 3)
        self.assertEqual(updates['body_html'], 'This article has slugline: Testing and dateline: '
                                               'hello world at 02 Jun 2015 08:53 AEST')
        self.assertListEqual(updates['place'], ['Australia'])

    def test_headline_strip_tags(self):
        template = {'data': {'headline': ' test\nit<br>'}}

        updates = render_content_template({}, template)
        self.assertEqual('test it', updates['headline'])

        item = get_item_from_template(template)
        self.assertEqual('test it', item['headline'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""NINJS formatter for Superdesk

.. versionadded:: 1.8
    Added *source* field.

.. versionadded:: 1.7
    Added *ednote* field.
    Added *signal* field.
    Added *genre* field.

.. versionchanged:: 1.7
    Fixed copyrightholder/copyrightnotice handling to be consistent with newsml.
    Fixed place property qcode should be code.
    Output profile name instead of _id in profile field.

.. versionadded:: 1.6
    Added *evolvedfrom* field to ninjs output.

"""


import json
import superdesk
import logging
import re

from eve.utils import config
from superdesk.publish.formatters import Formatter
from superdesk.errors import FormatterError
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO, GUID_FIELD, ASSOCIATIONS
from superdesk.metadata.packages import RESIDREF, GROUP_ID, GROUPS, ROOT_GROUP, REFS
from superdesk.utils import json_serialize_datetime_objectId
from superdesk.media.renditions import get_renditions_spec
from apps.archive.common import get_utc_schedule
from superdesk import text_utils

logger = logging.getLogger(__name__)
# this regex match the way custom media fields are put in associations (i.e. how the key
# is generated). This is legacy, and can hardly be changed without risking to break
# instances in production.
MEDIA_FIELD_RE = re.compile(r"(?P<field_id>\S+)--(?P<version>\d+)$")
EXTRA_ITEMS = "extra_items"


def filter_empty_vals(data):
    """Filter out `None` values from a given dict."""
    return dict(filter(lambda x: x[1], data.items()))


def get_locale_name(item, language):
    try:
        return item['translations']['name'][language]
    except (KeyError, TypeError):
        return item.get('name', '')


def format_cv_item(item, language):
    """Format item from controlled vocabulary for output."""
    return filter_empty_vals({
        'code': item.get('qcode'),
        'name': get_locale_name(item, language),
        'scheme': item.get('scheme')
    })


class NINJSFormatter(Formatter):
    """
    The schema we use for the ninjs format is an extension
    of `the standard ninjs schema <http://www.iptc.org/std/ninjs/ninjs-schema_1.1.json>`_.

    *Changes from ninjs schema*:

    * ``uri`` was replaced by ``guid``: ``uri`` should be the resource identifier on the web
        but since the item was not published yet it can't be determined at this point
    * added ``priority`` field
    * added ``service`` field
    * added ``slugline`` field
    * added ``keywords`` field
    * added ``evolvedfrom`` field
    * added ``source`` field

    Associations dictionary may contain entire items like
    in `ninjs example <http://dev.iptc.org/ninjs-Examples-3>`_ or just the item ``guid``
    and ``type``. In the latest case the items are sent separately before the package item.
    """

    direct_copy_properties = ('versioncreated', 'usageterms', 'language', 'headline', 'copyrightnotice',
                              'urgency', 'pubstatus', 'mimetype', 'copyrightholder', 'ednote',
                              'body_text', 'body_html', 'slugline', 'keywords',
                              'firstcreated', 'firstpublished', 'source', 'extra', 'annotations')

    rendition_properties = ('href', 'width', 'height', 'mimetype', 'poi', 'media')
    vidible_fields = {field: field for field in rendition_properties}
    vidible_fields.update({
        'url': 'href',
        'duration': 'duration',
        'mimeType': 'mimetype',
        'size': 'size',
    })

    def __init__(self):
        self.format_type = 'ninjs'
        self.can_preview = True
        self.can_export = True

    def format(self, article, subscriber, codes=None):
        try:
            pub_seq_num = superdesk.get_resource_service('subscribers').generate_sequence_number(subscriber)

            ninjs = self._transform_to_ninjs(article, subscriber)
            return [(pub_seq_num, json.dumps(ninjs, default=json_serialize_datetime_objectId))]
        except Exception as ex:
            raise FormatterError.ninjsFormatterError(ex, subscriber)

    def _transform_to_ninjs(self, article, subscriber, recursive=True):
        ninjs = {
            'guid': article.get(GUID_FIELD, article.get('uri')),
            'version': str(article.get(config.VERSION, 1)),
            'type': self._get_type(article)
        }

        if article.get('byline'):
            ninjs['byline'] = article['byline']

        located = article.get('dateline', {}).get('located', {})
        if located:
            ninjs['located'] = located.get('city', '')

        for copy_property in self.direct_copy_properties:
            if article.get(copy_property) is not None:
                ninjs[copy_property] = article[copy_property]

        if 'body_text' not in article and 'alt_text' in article:
            ninjs['body_text'] = article['alt_text']

        if 'title' in article:
            ninjs['headline'] = article['title']

        if article.get('body_html'):
            ninjs['body_html'] = self.append_body_footer(article)

        if article.get('description'):
            ninjs['description_html'] = self.append_body_footer(article)

        if article.get('place'):
            ninjs['place'] = self._format_place(article)

        if article.get('profile'):
            ninjs['profile'] = self._format_profile(article['profile'])

        extra_items = None
        if recursive:
            if article[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
                ninjs[ASSOCIATIONS] = self._get_associations(article, subscriber)
                if article.get(ASSOCIATIONS):
                    associations, extra_items = self._format_related(article, subscriber)
                    ninjs[ASSOCIATIONS].update(associations)
            elif article.get(ASSOCIATIONS):
                ninjs[ASSOCIATIONS], extra_items = self._format_related(article, subscriber)
        elif article.get(ASSOCIATIONS):
            ninjs[ASSOCIATIONS], extra_items = self._format_related(article, subscriber)
        if extra_items:
            ninjs.setdefault(EXTRA_ITEMS, {}).update(extra_items)

        if article.get(EMBARGO):
            ninjs['embargoed'] = get_utc_schedule(article, EMBARGO).isoformat()

        if article.get('priority'):
            ninjs['priority'] = article['priority']
        else:
            ninjs['priority'] = 5

        if article.get('subject'):
            ninjs['subject'] = self._get_subject(article)

        if article.get('anpa_category'):
            ninjs['service'] = self._get_service(article)
        if article.get('renditions'):
            ninjs['renditions'] = self._get_renditions(article)
        elif 'url' in article:
            ninjs['renditions'] = self._generate_renditions(article)

        # SDPA-317
        if 'abstract' in article:
            abstract = article.get('abstract', '')
            ninjs['description_html'] = abstract
            ninjs['description_text'] = text_utils.get_text(abstract)
        elif article.get('description_text'):
            ninjs['description_text'] = article.get('description_text')

        if article.get('company_codes'):
            ninjs['organisation'] = [{'name': c.get('name', ''), 'rel': 'Securities Identifier',
                                      'symbols': [{'ticker': c.get('qcode', ''),
                                                   'exchange': c.get('security_exchange', '')}]}
                                     for c in article['company_codes']]
        elif 'company' in article:
            ninjs['organisation'] = [{'name': article['company']}]

        if article.get('rewrite_of'):
            ninjs['evolvedfrom'] = article['rewrite_of']

        if not ninjs.get('copyrightholder') and not ninjs.get('copyrightnotice') and not ninjs.get('usageterms'):
            ninjs.update(superdesk.get_resource_service('vocabularies').get_rightsinfo(article))

        if 'genre' in article:
            ninjs['genre'] = self._get_genre(article)

        if article.get('flags', {}).get('marked_for_legal'):
            ninjs['signal'] = self._format_signal_cwarn()

        if article.get('attachments'):
            ninjs['attachments'] = self._format_attachments(article)

        if ninjs['type'] == CONTENT_TYPE.TEXT and ('body_html' in ninjs or 'body_text' in ninjs):
            if 'body_html' in ninjs:
                body_html = ninjs['body_html']
                word_count = text_utils.get_word_count(body_html)
                char_count = text_utils.get_char_count(body_html)
                readtime = text_utils.get_reading_time(body_html, word_count, article.get('language'))
            else:
                body_text = ninjs['body_text']
                word_count = text_utils.get_text_word_count(body_text)
                char_count = len(body_text)
                readtime = text_utils.get_reading_time(body_text, word_count, article.get('language'))
            ninjs['charcount'] = char_count
            ninjs['wordcount'] = word_count
            ninjs['readtime'] = readtime

        if article.get('authors'):
            ninjs['authors'] = self._format_authors(article)

        return ninjs

    def _generate_renditions(self, article):
        """
        For associated items that have custom structure generate renditions based on the item `custom properties.
        """
        renditions = {'original': {}}
        for orig_field, dest_field in self.vidible_fields.items():
            if orig_field in article:
                renditions['original'][dest_field] = article[orig_field]
        if 'thumbnail' in article:
            renditions['thumbnail'] = {'href': article['thumbnail']}
        return renditions

    def can_format(self, format_type, article):
        return format_type == self.format_type

    def _get_type(self, article):
        if article[ITEM_TYPE] == CONTENT_TYPE.PREFORMATTED:
            return CONTENT_TYPE.TEXT
        return article[ITEM_TYPE]

    def _get_associations(self, article, subscriber):
        """Create associations dict for package groups."""
        associations = dict()
        for group in article.get(GROUPS, []):
            if group[GROUP_ID] == ROOT_GROUP:
                continue

            group_items = []
            for ref in group[REFS]:
                if RESIDREF in ref:
                    item = {}
                    item['guid'] = ref[RESIDREF]
                    item[ITEM_TYPE] = ref.get(ITEM_TYPE, 'text')
                    if 'label' in ref:
                        item['label'] = ref.get('label')
                    if ref.get('package_item'):
                        item.update(self._transform_to_ninjs(ref['package_item'], subscriber, recursive=False))
                    group_items.append(item)
            if len(group_items) == 1:
                associations[group[GROUP_ID]] = group_items[0]
            elif len(group_items) > 1:
                for index in range(0, len(group_items)):
                    associations[group[GROUP_ID] + '-' + str(index)] = group_items[index]
        return associations

    def _format_related(self, article, subscriber):
        """Format all associated items for simple items (not packages)."""
        associations = {}
        extra_items = {}
        media = {}
        content_profile = None
        for key, item in (article.get(ASSOCIATIONS) or {}).items():
            if item:
                item = self._transform_to_ninjs(item, subscriber)
                associations[key] = item  # all items should stay in associations
                match = MEDIA_FIELD_RE.match(key)
                if match:
                    # item id seems to be build from a custom id
                    # we now check content profile to see if it correspond to a custom field
                    if content_profile is None:
                        try:
                            profile = article['profile']
                        except KeyError:
                            logger.warning("missing profile in article (guid: {guid})".format(guid=article.get("guid")))
                            content_profile = {"schema": {}}
                        else:
                            content_profile = superdesk.get_resource_service("content_types").find_one(
                                _id=profile, req=None)
                    field_id = match.group("field_id")
                    schema = content_profile['schema'].get(field_id, {})
                    if schema.get("type") == "media":
                        # we want custom media fields in "extra_items", cf. SDESK-2955
                        version = match.group("version")
                        media.setdefault(field_id, []).append((version, item))

        if media:
            # we have custom media fields, we now order them
            # and add them to "extra_items"
            for field_id, data in media.items():
                extra_items[field_id] = {"items": [d[1] for d in sorted(data)]}

        return associations, extra_items

    def _get_genre(self, article):
        lang = article.get('language', '')
        return [format_cv_item(item, lang) for item in article['genre']]

    def _get_subject(self, article):
        """Get subject list for article."""
        return [format_cv_item(item, article.get('language', '')) for item in article.get('subject', [])]

    def _get_service(self, article):
        """Get service list for article.

        It's using `anpa_category` to populate service field for now.
        """
        return [format_cv_item(item, article.get('language', '')) for item in article.get('anpa_category', [])]

    def _get_renditions(self, article):
        """Get renditions for article."""
        # get the actual article's renditions
        actual_renditions = article.get('renditions', {})
        # renditions list that we want to publish
        if article['type'] is 'picture':
            renditions_to_publish = ['original'] + list(get_renditions_spec(without_internal_renditions=True).keys())
            # filter renditions and keep only the ones we want to publish
            actual_renditions = {name: actual_renditions[name] for name in renditions_to_publish
                                 if name in actual_renditions}
        # format renditions to Ninjs
        renditions = {}
        for name, rendition in actual_renditions.items():
            renditions[name] = self._format_rendition(rendition)
        return renditions

    def _format_rendition(self, rendition):
        """Format single rendition using fields whitelist."""
        return {field: rendition[field] for field in self.rendition_properties if field in rendition}

    def _format_place(self, article):
        vocabularies_service = superdesk.get_resource_service('vocabularies')
        locator_map = vocabularies_service.find_one(req=None, _id='locators')
        if locator_map and 'items' in locator_map:
            locator_map['items'] = vocabularies_service.get_locale_vocabulary(
                locator_map.get('items'), article.get('language'))

        def get_label(item):
            if locator_map:
                locators = [l for l in locator_map.get('items', []) if l['qcode'] == item.get('qcode')]
                if locators and len(locators) == 1:
                    return locators[0].get('state') or \
                        locators[0].get('country') or \
                        locators[0].get('world_region') or \
                        locators[0].get('group')
            return item.get('name')

        places = []
        for item in article.get('place', []):
            if item.get('scheme') == 'geonames':
                places.append(self._format_geonames(item))
            else:
                places.append({'name': get_label(item), 'code': item.get('qcode')})
        return places

    def _format_geonames(self, place):
        return {
            'scheme': place['scheme'],
            'code': place['code'],
            'name': place['name'],
        }

    def _format_profile(self, profile):
        return superdesk.get_resource_service('content_types').get_output_name(profile)

    def _format_signal_cwarn(self):
        return [{'name': 'Content Warning', 'code': 'cwarn', 'scheme': 'http://cv.iptc.org/newscodes/signal/'}]

    def _format_attachments(self, article):
        output = []
        attachments_service = superdesk.get_resource_service('attachments')
        for attachment_ref in article['attachments']:
            attachment = attachments_service.find_one(req=None, _id=attachment_ref['attachment'])
            output.append({
                'id': str(attachment['_id']),
                'title': attachment['title'],
                'description': attachment['description'],
                'filename': attachment['filename'],
                'mimetype': attachment['mimetype'],
                'length': attachment.get('length'),
                'media': str(attachment['media']),
                'href': '/assets/{}'.format(str(attachment['media'])),
            })
        return output

    def _format_authors(self, article):
        users_service = superdesk.get_resource_service('users')
        vocabularies_service = superdesk.get_resource_service('vocabularies')
        job_titles_voc = vocabularies_service.find_one(None, _id='job_titles')
        if job_titles_voc and 'items' in job_titles_voc:
            job_titles_voc['items'] = vocabularies_service.get_locale_vocabulary(
                job_titles_voc.get('items'), article.get('language'))
        job_titles_map = {v['qcode']: v['name'] for v in job_titles_voc['items']} if job_titles_voc is not None else {}

        authors = []
        for author in article['authors']:
            try:
                user_id = author['parent']
            except KeyError:
                # XXX: in some older items, parent may be missing, we try to find user with name in this case
                try:
                    user = next(users_service.find({'display_name': author['name']}))
                except (StopIteration, KeyError):
                    logger.warning("unknown user")
                    user = {}
            else:
                try:
                    user = next(users_service.find({'_id': user_id}))
                except StopIteration:
                    logger.warning("unknown user: {user_id}".format(user_id=user_id))
                    user = {}

            avatar_url = user.get('picture_url', author.get('avatar_url'))

            author = {
                "name": user.get('display_name', author.get('name', '')),
                "role": author.get('role', ''),
                "biography": user.get('biography', author.get('biography', ''))
            }

            # include socials only if they are non-empty
            socials = ('facebook', 'twitter', 'instagram')
            for social in socials:
                social_data = user.get(social, author.get(social, ''))
                if social_data:
                    author[social] = social_data

            if avatar_url:
                author['avatar_url'] = avatar_url

            job_title_qcode = user.get('job_title')
            if job_title_qcode is not None:
                author['jobtitle'] = {'qcode': job_title_qcode,
                                      'name': job_titles_map.get(job_title_qcode, '')}

            authors.append(author)
        return authors

    def export(self, item):
        if self.can_format(self.format_type, item):
            sequence, formatted_doc = self.format(item, {'_id': '0'}, None)[0]
            return formatted_doc.replace('\'\'', '\'')
        else:
            raise Exception()




import json

import apps.archive  # NOQA
# XXX: This import is needed in order to avoid ImportError when importing
# package_service, caused by circular dependencies.
# When that issue is resolved, the workaround should be removed.

import apps.packages.package_service as package
from flask import current_app as app
from superdesk import get_resource_service
from superdesk.services import BaseService
from eve.utils import ParsedRequest
from superdesk.notification import push_notification
from superdesk.utc import get_timezone_offset, utcnow
from eve.utils import config
from apps.archive.common import ITEM_MARK, ITEM_UNMARK


def init_parsed_request(elastic_query):
    parsed_request = ParsedRequest()
    parsed_request.args = {"source": json.dumps(elastic_query)}
    return parsed_request


def get_highlighted_items(highlights_id):
    """Get items marked for given highlight and passing date range query."""
    highlight = get_resource_service('highlights').find_one(req=None, _id=highlights_id)
    query = {
        'query': {
            'filtered': {'filter': {'and': [
                {'range': {'versioncreated': {'gte': highlight.get('auto_insert', 'now/d'),
                                              'time_zone': get_timezone_offset(config.DEFAULT_TIMEZONE,
                                                                               utcnow())}}},
                {'term': {'highlights': str(highlights_id)}},
            ]}}
        },
        'sort': [
            {'versioncreated': 'desc'},
        ],
        'size': 200
    }
    request = ParsedRequest()
    request.args = {'source': json.dumps(query), 'repo': 'archive,published'}
    return list(get_resource_service('search').get(req=request, lookup=None))


def init_highlight_package(doc):
    """Add to package items marked for doc highlight."""
    main_group = doc.get('groups')[1]
    items = get_highlighted_items(doc.get('highlight'))
    used_items = []
    for item in items:
        if item['_id'] not in used_items:
            main_group['refs'].append(package.get_item_ref(item))
            used_items.append(item['_id'])


def on_create_package(sender, docs):
    """Call init_highlight_package for each package with highlight reference."""
    for doc in docs:
        if doc.get('highlight'):
            init_highlight_package(doc)


def get_highlight_name(highlights_id):
    """
    Given the id of a highlight it will return the name.
    :param hightlight_id:
    :return:
    """
    highlight = get_resource_service('highlights').find_one(req=None, _id=highlights_id)
    if highlight and 'name' in highlight:
        return highlight.get('name', None)
    return None


class HighlightsService(BaseService):
    def on_delete(self, doc):
        service = get_resource_service('archive')
        highlights_id = str(doc['_id'])
        query = {'query': {'filtered': {'filter': {'term': {'highlights': highlights_id}}}}}
        req = init_parsed_request(query)
        proposedItems = service.get(req=req, lookup=None)
        for item in proposedItems:
            app.on_archive_item_updated(
                {'highlight_id': highlights_id, 'highlight_name': get_highlight_name(highlights_id)}, item, ITEM_UNMARK)
            highlights = [h for h in item.get('highlights') if h != highlights_id]
            service.update(item['_id'], {'highlights': highlights}, item)


class MarkedForHighlightsService(BaseService):
    def create(self, docs, **kwargs):
        """Toggle highlight status for given highlight and item."""
        service = get_resource_service('archive')
        publishedService = get_resource_service('published')
        ids = []
        for doc in docs:
            item = service.find_one(req=None, _id=doc['marked_item'])
            if not item:
                ids.append(None)
                continue
            ids.append(item['_id'])
            highlights = item.get('highlights', [])
            if not highlights:
                highlights = []

            if doc['highlights'] not in highlights:
                highlights.append(doc['highlights'])
                highlight_on = True  # highlight toggled on
            else:
                highlights = [h for h in highlights if h != doc['highlights']]
                highlight_on = False  # highlight toggled off

            updates = {
                'highlights': highlights,
                '_updated': item['_updated'],
                '_etag': item['_etag']
            }
            service.update(item['_id'], updates, item)

            publishedItems = publishedService.find({'item_id': item['_id']})
            for publishedItem in publishedItems:
                if publishedItem['_current_version'] == item['_current_version'] or not highlight_on:
                    updates = {
                        'highlights': highlights,
                        '_updated': publishedItem['_updated'],
                        '_etag': publishedItem['_etag']
                    }
                    publishedService.update(publishedItem['_id'], updates, publishedItem)

            if highlight_on:
                app.on_archive_item_updated(
                    {'highlight_id': doc['highlights'], 'highlight_name': get_highlight_name(doc['highlights'])}, item,
                    ITEM_MARK)
            else:
                app.on_archive_item_updated(
                    {'highlight_id': doc['highlights'], 'highlight_name': get_highlight_name(doc['highlights'])}, item,
                    ITEM_UNMARK)

            push_notification(
                'item:highlights',
                marked=int(highlight_on),
                item_id=item['_id'],
                mark_id=str(doc['highlights']))

        return ids


package.package_create_signal.connect(on_create_package)




from superdesk.tests import TestCase
from apps.archive.commands import RemoveExpiredContent


class RemoveExpiredContentTestCase(TestCase):

    def test_is_expired(self):
        command = RemoveExpiredContent()
        item = {'expiry': None, '_id': 'foo'}
        self.assertFalse(command._can_remove_item(item))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os

from superdesk.io.feed_parsers.dpa_iptc7901 import DPAIPTC7901FeedParser
from superdesk.tests import TestCase


def fixture(filename):
    dirname = os.path.dirname(os.path.realpath(__file__))
    return os.path.normpath(os.path.join(dirname, '../fixtures', filename))


class DPAIptcTestCase(TestCase):

    parser = DPAIPTC7901FeedParser()

    def open(self, filename):
        provider = {'name': 'Test'}
        return self.parser.parse(fixture(filename), provider)

    def test_open_iptc7901_file(self):
        with self.app.app_context():
            item = self.open('IPTC7901.txt')
            self.assertEqual('text', item['type'])
            self.assertEqual('062', item['ingest_provider_sequence'])
            self.assertEqual('i', item['anpa_category'][0]['qcode'])
            self.assertEqual(211, item['word_count'])
            self.assertEqual('Germany Social Democrats: Coalition talks with Merkel could fail =', item['headline'])
            self.assertRegex(item['body_html'], '^<p></p><p>Negotiations')
            self.assertEqual('Germany-politics', item['slugline'])
            self.assertEqual(4, item['priority'])
            self.assertEqual([{'qcode': 'i'}], item['anpa_category'])
            self.assertTrue(item['ednote'].find('## Editorial contacts'))
            self.assertEqual(item['dateline']['source'], 'dpa')
            self.assertEqual(item['dateline']['located']['city'], 'Berlin')

    def test_open_dpa_copyright(self):
        with self.app.app_context():
            item = self.open('dpa_copyright.txt')
            self.assertEqual('text', item['type'])
            self.assertEqual('rs', item['anpa_category'][0]['qcode'])
            self.assertEqual('(Achtung)', item['headline'])
            self.assertEqual('Impressum', item['slugline'])




import redis
import hermes
import hermes.backend
import hermes.backend.dict
import hermes.backend.redis

from flask import current_app as app
from superdesk import json_utils
from superdesk.logging import logger


class SuperdeskRedisBackend(hermes.backend.redis.Backend):
    """Updated init to create redis from URL instead of parsing params."""

    def __init__(self, mangler, **kwargs):
        self.mangler = mangler
        self.client = redis.StrictRedis.from_url(kwargs.pop('url'))
        self._options = kwargs


class SuperdeskMangler(hermes.Mangler):
    """Implements encoding/decoding for Superdesk data - so handles ObjectIds, dates etc."""

    def hash(self, value):
        try:
            encoded_value = value.encode('utf-8')
        except AttributeError:
            encoded_value = value
        return super().hash(encoded_value)

    def dumps(self, value):
        return json_utils.dumps(value)

    def loads(self, value):
        return json_utils.loads(value)


class SuperdeskCacheBackend(hermes.backend.AbstractBackend):
    """Proxy for hermes cache backend.

    It will only initialize proper cache backend when some cache method is called,
    so we can use @cache decorator before the app starts.

    Later it reads ``CACHE_URL`` config to figure out if we want to use redis backend
    or memcached.
    """

    @property
    def _backend(self):
        if not app:
            raise RuntimeError('You can only use cache within app context.')

        if not app.cache:
            cache_url = app.config.get('CACHE_URL', '')
            if 'redis' in cache_url or 'unix' in cache_url:
                app.cache = SuperdeskRedisBackend(self.mangler, url=cache_url)
                logger.info('using redis cache backend')
            elif cache_url:
                import hermes.backend.memcached
                app.cache = hermes.backend.memcached.Backend(self.mangler, servers=[cache_url])
                logger.info('using memcached cache backend')
            else:
                app.cache = hermes.backend.dict.Backend(self.mangler)
                logger.info('using dict cache backend')

        return app.cache

    def lock(self, key):
        return self._backend.lock(key)

    def save(self, key=None, value=None, mapping=None, ttl=None):
        return self._backend.save(key, value, mapping, ttl)

    def load(self, keys):
        val = self._backend.load(keys)
        return val

    def remove(self, keys):
        return self._backend.remove(keys)

    def clean(self):
        return self._backend.clean()


cache = hermes.Hermes(SuperdeskCacheBackend, SuperdeskMangler, ttl=600)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource, build_custom_hateoas
from superdesk.services import BaseService
from .common import CUSTOM_HATEOAS
from superdesk.metadata.utils import aggregations
from .archive import ArchiveResource
import superdesk


class UserContentResource(Resource):
    endpoint_name = 'user_content'
    item_url = ArchiveResource.item_url
    url = 'users/<regex("[a-f0-9]{24}"):original_creator>/content'
    schema = ArchiveResource.schema
    datasource = {
        'source': 'archive',
        'aggregations': aggregations,
        'elastic_filter': {
            'and': [
                {'not': {'exists': {'field': 'task.desk'}}},
                {'not': {'term': {'version': 0}}},
            ]
        }
    }
    resource_methods = ['GET', 'POST']
    item_methods = ['GET', 'PATCH', 'DELETE']
    resource_title = endpoint_name


class UserContentService(BaseService):

    def get(self, req, lookup):
        docs = super().get(req, lookup)
        for doc in docs:
            build_custom_hateoas(CUSTOM_HATEOAS, doc)
        return docs


superdesk.workflow_state('draft')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock
from unittest.mock import MagicMock

from superdesk.tests import TestCase


class MarkedForHighlightsServiceTest(TestCase):
    """Base class for MarkedForHighlightsService tests."""

    def setUpForChildren(self):
        super().setUpForChildren()

        from apps.highlights import init_app
        # with self.app.app_context():
        init_app(self.app)

        try:
            from apps.highlights.service import MarkedForHighlightsService
        except ImportError:
            self.fail(
                "Could not import class under test "
                "(MarkedForHighlightsService).")
        else:
            self.instance = MarkedForHighlightsService()


@mock.patch('apps.highlights.service.push_notification')
@mock.patch('apps.highlights.service.get_resource_service')
class CreateMethodTestCase(MarkedForHighlightsServiceTest):
    """Tests for the create() method."""

    def setUp(self):
        db_item_1 = {
            '_id': 'tag:item_1',
            'highlights': [],
            '_updated': '01.01.1111',
            '_etag': '111',
        }
        db_item_2 = {
            '_id': 'tag:item_2',
            'highlights': [],
            '_updated': '02.02.2222',
            '_etag': '222',
        }

        # some of the existing archive items in database
        self.db_items = {
            'tag:item_1': db_item_1,
            'tag:item_2': db_item_2
        }

    def test_pushes_notifications_for_newly_highlighted_items(
        self, fake_get_service, fake_push_notify
    ):

        def fake_find_one(**kwargs):
            item_id = kwargs.get('_id')
            return self.db_items.get(item_id)

        fake_archive_service = MagicMock()
        fake_archive_service.find_one = fake_find_one

        fake_get_service.return_value = fake_archive_service

        # items' data in HTTP request
        req_item_1 = {
            'marked_item': 'tag:item_1',
            'highlights': 'highlight_X'
        }
        req_item_2 = {
            'marked_item': 'tag:item_2',
            'highlights': 'highlight_Y'
        }

        self.instance.create([req_item_1, req_item_2])

        # notifications should have been pushed, one for each highlighted item
        self.assertEqual(fake_push_notify.call_count, 2)
        fake_push_notify.assert_any_call(
            'item:highlights',
            marked=1,
            item_id='tag:item_1',
            mark_id='highlight_X'
        )
        fake_push_notify.assert_any_call(
            'item:highlights',
            marked=1,
            item_id='tag:item_2',
            mark_id='highlight_Y'
        )

    def test_pushes_notifications_for_newly_unhighlighted_items(
        self, fake_get_service, fake_push_notify
    ):
        # existing items ARE highlighted, and we will test toggling this off
        self.db_items['tag:item_1']['highlights'] = ['highlight_X']
        self.db_items['tag:item_2']['highlights'] = ['highlight_Y']

        def fake_find_one(**kwargs):
            item_id = kwargs.get('_id')
            return self.db_items.get(item_id)

        fake_archive_service = MagicMock()
        fake_archive_service.find_one = fake_find_one

        fake_get_service.return_value = fake_archive_service

        # items' data in HTTP request
        req_item_1 = {
            'marked_item': 'tag:item_1',
            'highlights': 'highlight_X'
        }
        req_item_2 = {
            'marked_item': 'tag:item_2',
            'highlights': 'highlight_Y'
        }

        self.instance.create([req_item_1, req_item_2])

        # notifications should have been pushed, one for each highlighted item
        self.assertEqual(fake_push_notify.call_count, 2)
        fake_push_notify.assert_any_call(
            'item:highlights',
            marked=0,
            item_id='tag:item_1',
            mark_id='highlight_X'
        )
        fake_push_notify.assert_any_call(
            'item:highlights',
            marked=0,
            item_id='tag:item_2',
            mark_id='highlight_Y'
        )



import json
import logging
import os
import re
from collections import OrderedDict
from pathlib import Path

import superdesk
import pymongo
from eve.utils import config
from flask import current_app as app

logger = logging.getLogger(__name__)

"""
App initialization information, maps resource name to the file containing the data
and the index to be created for the resource and the boolean flag to update the
resource or not.
__entities__ = {
    "resource_name": ("file_name", "index_params", "do_patch")
}
"file_name" (str): name of the file containing seed data
"index_params list: List of key (field or List of tuple as required by pymongo create_index function.
http://api.mongodb.org/python/current/api/pymongo/collection.html
For example:
[[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], "username"] will create two indexes
- composite index of "first_name", "last_name" field.
- index on username field.
Alternatively index param can be specified as
[[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], [("username", pymongo.ASCENDING)]]
Options can be sent to index creation and in this case the last element in the list is the options
dictionary:
[[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], {'sparse': True}]]
"""
__entities__ = OrderedDict([
    ('roles', ('roles.json', ['name'], False)),
    ('users', ('users.json', [], False)),
    ('stages', ('stages.json', ['desk'], False)),
    ('desks', ('desks.json', ['incoming_stage'], False)),
    ('vocabularies', ('vocabularies.json', '', True)),
    ('validators', ('validators.json', '', True)),
    ('content_templates', ('content_templates.json', [
        [('template_name', pymongo.ASCENDING)],
        [('next_run', pymongo.ASCENDING)],
    ], False)),
    ('content_types', ('content_types.json', '', True)),
    ('published', (None, [
        [
            ('expiry', pymongo.ASCENDING),
            ('_created', pymongo.ASCENDING),
            ('state', pymongo.ASCENDING)
        ],
        [
            ('item_id', pymongo.ASCENDING),
            ('state', pymongo.ASCENDING)
        ],
        [
            ('publish_sequence_no', pymongo.ASCENDING)
        ],
        [
            ('queue_state', pymongo.ASCENDING)
        ]], False)),
    ('activity', (None, [
        [
            ('_created', pymongo.DESCENDING),
            {'expireAfterSeconds': 86400}
        ],
        [
            ('user', pymongo.ASCENDING),
            ('_created', pymongo.DESCENDING),
        ],
        [
            ('item', pymongo.ASCENDING),
            ('read', pymongo.ASCENDING),
            ('user', pymongo.ASCENDING)
        ],
        [
            ('resource', pymongo.ASCENDING),
            ('data.provider_id', pymongo.ASCENDING)
        ]], False)),
    ('archive', (None, [
        [
            ('_updated', pymongo.ASCENDING)
        ],
        [
            ('expiry', pymongo.ASCENDING),
            ('state', pymongo.ASCENDING)
        ],
        [
            ('type', pymongo.ASCENDING)
        ],
        [
            ('groups.refs.residRef', pymongo.ASCENDING),
            {'sparse': True}
        ],
        [
            ('schedule_settings.utc_publish_schedule', pymongo.ASCENDING),
            ('state', pymongo.ASCENDING)
        ],
        [
            ('unique_name', pymongo.ASCENDING)
        ]], False)),
    ('archive_versions', (None, [
        [
            ('_id_document', pymongo.ASCENDING),
            ('_current_version', pymongo.ASCENDING)
        ]], False)),
    ('ingest', (None, [
        [
            ('expiry', pymongo.ASCENDING),
            ('ingest_provider', pymongo.ASCENDING)
        ],
        [
            ('guid', pymongo.ASCENDING)
        ]], False)),
    ('publish_queue', (None, [
        [
            ('_created', pymongo.DESCENDING),
            ('state', pymongo.ASCENDING),
            ('destination.delivery_type', pymongo.ASCENDING)
        ],
        [
            ('item_id', pymongo.ASCENDING),
            ('item_version', pymongo.ASCENDING)
        ],
        [
            ('state', pymongo.ASCENDING),
            ('destination.delivery_type', pymongo.ASCENDING)
        ],
        [
            ('subscriber_id', pymongo.ASCENDING)
        ],
        [
            ('_updated', pymongo.DESCENDING)
        ],
        [
            ('ingest_provider', pymongo.ASCENDING)
        ]], False)),
    ('archived', (None, [
        [
            ('archived_id', pymongo.ASCENDING),
            {'unique': True}
        ],
        [
            ('item_id', pymongo.ASCENDING)
        ],
    ], False)),

    ('legal_archive', (None, [
        [('versioncreated', pymongo.DESCENDING)],
    ], False)),

    ('legal_archive_versions', (None, [
        [
            ('_id_document', pymongo.ASCENDING),
            ('_current_version', pymongo.ASCENDING)
        ]], False)),
    ('legal_publish_queue', (None, [
        [('_updated', pymongo.DESCENDING)]
    ], False)),
    ('dictionaries', ('dictionaries.json', '', True)),
    ('ingest_providers', ('ingest_providers.json', '', True)),
    ('search_providers', ('search_providers.json', '', True)),
    ('products', ('products.json', '', True)),
    ('subscribers', ('subscribers.json', '', True)),
    ('workspaces', ('workspaces.json', '', True)),
    ('item_comments', (None, [
        [('item', pymongo.ASCENDING), ('_created', pymongo.DESCENDING)]
    ], True)),
    ('audit', (None, [[('_updated', pymongo.ASCENDING)],
                      [('_id', pymongo.ASCENDING), ('_updated', pymongo.ASCENDING)]], False)),
    ('contacts', ('contacts.json', '', False)),
    ('planning_types', ('planning_types.json', '', True))
])
INIT_DATA_PATH = Path(__file__).resolve().parent / 'data_init'


def get_filepath(filename, path=None):
    """Get filepath for filename.

    If path is set, only check that. Otherwise use configured INIT_DATA_PATH,
    and if the file is not there, check default data path in core.

    :param filename: filename
    :param path: custom path
    """
    if path:
        dirs = [path]
    else:
        dirs = [
            app.config.get('INIT_DATA_PATH', INIT_DATA_PATH),
            INIT_DATA_PATH,
        ]

    for basedir in dirs:
        filepath = Path(basedir) / filename
        if filepath.exists():
            return filepath


class AppInitializeWithDataCommand(superdesk.Command):
    """Initialize application with predefined data for various entities.

    Entities supported: [roles, users, desks, stages, vocabularies, validators, content_templates, content_types,
    published, activity, archive, archive_versions, ingest, publish_queue, archived, legal_archive,
    legal_archive_versions, legal_publish_queue, dictionaries, ingest_providers, search_providers, products,
    subscribers, workspaces, item_comments, audit, contacts, planning_types].
    If no --entity-name parameter is supplied, all the entities are inserted.
    The entities [vocabularies, validators, content_types, dictionaries, ingest_providers, search_providers,
    products, subscribers, workspaces, item_comments, planning_types] will be updated with the predefined data
    if it already exists, no action will be taken for the other entities.
    """

    option_list = [
        superdesk.Option('--entity-name', '-n', action='append'),
        superdesk.Option('--full-path', '-p', dest='path'),
        superdesk.Option('--sample-data', action='store_true'),
        superdesk.Option('--force', '-f', action='store_true'),
        superdesk.Option('--init-index-only', '-i', action='store_true'),
    ]

    def run(self, entity_name=None, path=None, sample_data=False, force=False, init_index_only=False):
        """Run the initialization

        :param str,list,NoneType entity_name: entity(ies) to initialize
        :param str,NoneType path: path of the file to import
        :param bool sample_data: True if sample data need to be used
        :param bool force: if True, update item even if it has been modified by user
        :param bool init_index_only: if True, it only initializes index only
        """
        logger.info('Starting data initialization')
        logger.info('Config: %s', app.config['APP_ABSPATH'])

        # create indexes in mongo
        app.init_indexes()
        # put mapping to elastic
        app.data.init_elastic(app)

        if init_index_only:
            logger.info('Only indexes initialized.')
            return 0

        if sample_data:
            if not path:
                path = INIT_DATA_PATH.parent / 'data_sample'
            else:
                raise ValueError('path and sample_data should not be set at the same time')

        if entity_name:
            if isinstance(entity_name, str):
                entity_name = [entity_name]
            for name in entity_name:
                (file_name, index_params, do_patch) = __entities__[name]
                self.import_file(name, path, file_name, index_params, do_patch, force)
            return 0

        for name, (file_name, index_params, do_patch) in __entities__.items():
            try:
                self.import_file(name, path, file_name, index_params, do_patch, force)
            except KeyError:
                continue
            except Exception as ex:
                logger.exception(ex)
                logger.info('Exception loading entity {} from {}'.format(name, file_name))

        logger.info('Data import finished')
        return 0

    def import_file(self, entity_name, path, file_name, index_params, do_patch=False, force=False):
        """Imports seed data based on the entity_name (resource name) from the file_name specified.

        index_params use to create index for that entity/resource

        :param str entity_name: name of the resource
        :param str file_name: file name that contains seed data
        :param list index_params: list of indexes that is created on that entity.
        For example:
        [[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], "username"] will create two indexes
        - composite index of "first_name", "last_name" field.
        - index on username field.
        Alternatively index param can be specified as
        [[("first_name", pymongo.ASCENDING), ("last_name", pymongo.ASCENDING)], [("username", pymongo.ASCENDING)]]
        Refer to pymongo create_index documentation for more information.
        http://api.mongodb.org/python/current/api/pymongo/collection.html
        :param bool do_patch: if True then patch the document else don't patch.
        """
        logger.info('Process %r', entity_name)
        file_path = file_name and get_filepath(file_name, path)
        if not file_path:
            pass
        elif not file_path.exists():
            logger.info(' - file not exists: %s', file_path)
        else:
            logger.info(' - got file path: %s', file_path)
            with file_path.open('rt', encoding='utf-8') as app_prepopulation:
                service = superdesk.get_resource_service(entity_name)
                json_data = json.loads(app_prepopulation.read())
                data = [fillEnvironmentVariables(item) for item in json_data]
                data = [app.data.mongo._mongotize(item, service.datasource) for item in data if item]
                existing_data = []
                existing = service.get_from_mongo(None, {})
                update_data = True
                if not do_patch and existing.count() > 0:
                    logger.info(' - data already exists none will be loaded')
                    update_data = False
                elif do_patch and existing.count() > 0:
                    logger.info(' - data already exists it will be updated')

                if update_data:
                    if do_patch:
                        for item in existing:
                            for loaded_item in data:
                                if '_id' in loaded_item and loaded_item['_id'] == item['_id']:
                                    data.remove(loaded_item)
                                    if force or item.get('init_version', 0) < loaded_item.get('init_version', 0):
                                        existing_data.append(loaded_item)

                    if data:
                        for item in data:
                            if not item.get(config.ETAG):
                                item.setdefault(config.ETAG, 'init')
                        service.post(data)

                    if existing_data and do_patch:
                        for item in existing_data:
                            item['_etag'] = 'init'
                            service.update(item['_id'], item, service.find_one(None, _id=item['_id']))

                logger.info(' - file imported successfully: %s', file_name)

        if index_params:
            for index in index_params:
                crt_index = list(index) if isinstance(index, list) else index
                options = crt_index.pop() if isinstance(crt_index[-1], dict) and isinstance(index, list) else {}
                collection = app.data.mongo.pymongo(resource=entity_name).db[entity_name]
                options['background'] = True
                index_name = collection.create_index(crt_index, **options)
                logger.info(' - index: %s for collection %s created successfully.', index_name, entity_name)


def fillEnvironmentVariables(item):
    variables = {}
    text = json.dumps(item)

    for variable in re.findall('#ENV_([^#"]+)#', text):
        value = os.environ.get(variable, None)
        if not value:
            return None
        else:
            variables[variable] = value

    for name in variables:
        text = text.replace('#ENV_%s#' % name, variables[name])

    return json.loads(text)


superdesk.command('app:initialize_data', AppInitializeWithDataCommand())



from .mimetype import Mimetype # noqa
from .preferences import Preferences # noqa
from .graphic import Graphic # noqa
from .styles import Styles # noqa
from .tags import Tags # noqa
from .spread import Spread # noqa
from .designmap import Designmap # noqa
from .stories import Story, StoryTable, StoryList # noqa
from .converter import Converter # noqa


# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.io.commands import update_ingest
from flask import request, abort
import os
import superdesk
import logging

logger = logging.getLogger(__name__)


class FeedinServiceWebhookAuth(object):

    def authenticate(self):
        abort(403, description='You are not authorized to access this resource')

    def authorized(self, allowed_roles, resource, method):
        try:
            # provider's id may be used in the future
            provider_name = request.args['provider_name']
        except KeyError:
            logger.warning('Got an invalid webhook request (missing provider name) ')
            return False
        # FIXME: there can be a conflict if 2 provider name are similar but with different case
        #        this will be fixed when we'll use id instead of name
        env_name = 'WEBHOOK_{}_AUTH'.format(provider_name.upper())
        try:
            auth_key = os.environ[env_name]
        except KeyError:
            logger.error('{} environment variable is not set'.format(env_name))
            return False
        return request.args.get('auth') == auth_key


class FeedingServiceWebhookResource(Resource):
    resource_methods = ['POST']
    authentication = FeedinServiceWebhookAuth
    # we don't want schema validation to accept any webhook data
    allow_unknown = True


class FeedingServiceWebhookService(BaseService):
    """Service givin metadata on backend itself"""

    def create(self, docs, **kwargs):
        # we don't want to create anything
        # we just use this service to trigget the provider
        # and return a fake id
        self.trigger_provider()
        return [0]

    def trigger_provider(self):
        provider_name = request.args['provider_name']
        lookup = {'name': provider_name}
        for provider in superdesk.get_resource_service('ingest_providers').get(req=None, lookup=lookup):
            kwargs = {
                'provider': provider,
                'rule_set': update_ingest.get_provider_rule_set(provider),
                'routing_scheme': update_ingest.get_provider_routing_scheme(provider)
            }
            update_ingest.update_provider.apply_async(
                expires=update_ingest.get_task_ttl(provider), kwargs=kwargs)


def init_app(app):
    service = FeedingServiceWebhookService()
    FeedingServiceWebhookResource("webhook", app=app, service=service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from superdesk import get_backend
from apps.marked_desks.resource import MarkedForDesksResource
from apps.marked_desks.service import MarkedForDesksService


def init_app(app):
    endpoint_name = 'marked_for_desks'
    service = MarkedForDesksService(endpoint_name, backend=get_backend())
    MarkedForDesksResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name='mark_for_desks', label='Mark items for desks',
                        description='User can mark items for other desks.')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from content_api.items.resource import ItemsResource


class PackagesResource(ItemsResource):
    """A class defining and configuring the /packages API endpoint."""

    datasource = {
        'source': 'items',
        'search_backend': 'elastic',
        'elastic_filter': {"bool": {"must": {"term": {"type": "composite"}}}},
        'default_sort': [('_updated', -1)],
    }
    item_methods = ['GET']
    resource_methods = ['GET']



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import hmac
import logging
import requests

from superdesk import app
from superdesk.publish import register_transmitter

from superdesk.errors import PublishHTTPPushError, PublishHTTPPushServerError, PublishHTTPPushClientError
from superdesk.publish.publish_queue import PUBLISHED_IN_PACKAGE
from superdesk.publish.publish_service import PublishService

errors = [PublishHTTPPushError.httpPushError().get_error_description()]
logger = logging.getLogger(__name__)


class HTTPPushService(PublishService):
    """HTTP Publish Service.

    The HTTP push service publishes items to the resource service via ``POST`` request.
    For media items it first publishes the media files to the assets service.

    For text items the publish sequence is like this:

    * ``POST`` to resource service the text item

    For media items the publish sequence is like this:

    * Publish media files: for each file from renditions perform the following steps:

        * Verify if the rendition media file exists in the assets service (``GET assets/{media_id}``)
        * If not, upload the rendition media file to the assets service via ``POST`` request

    * Publish the item

    For package items with embedded items config on there is only one publish request to the resource service.

    For package items without embedded items the publish sequence is like this:

    * Publish package items
    * Publish the package item

    **Publishing assets**

    The ``POST`` request to the assets ``URL`` has the ``multipart/data-form`` content type and should
    contain the following fields:

    ``media_id``
        URI string identifying the rendition.

    ``media``
        ``base64`` encoded file content. See `Eve documentation <http://python-eve.org/features.html#file-storage>`_.

    ``mime_type``
        mime type, eg. ``image/jpeg``.

    ``filemeta``
        metadata extracted from binary. Differs based on binary type, eg. could be exif for pictures.

    The response status code is checked - on success it should be ``201 Created``.
    If secret_token is set for destination the x-superdesk-signature header will be added
    for both json and multipart POST requests.
    """

    headers = {"Content-type": "application/json", "Accept": "application/json"}
    hash_header = 'x-superdesk-signature'

    def _transmit(self, queue_item, subscriber):
        """
        @see: PublishService._transmit
        """
        item = json.loads(queue_item['formatted_item'])
        destination = queue_item.get('destination', {})

        self._copy_published_media_files(json.loads(queue_item['formatted_item']), destination)

        if not queue_item.get(PUBLISHED_IN_PACKAGE) or not destination.get('config', {}).get('packaged', False):
            self._push_item(destination, json.dumps(item))

    def _push_item(self, destination, data):
        resource_url = self._get_resource_url(destination)
        headers = self._get_headers(data, destination, self.headers)
        response = requests.post(resource_url, data=data, headers=headers)

        # need to rethrow exception as a superdesk exception for now for notifiers.
        try:
            response.raise_for_status()
        except Exception as ex:
            logger.exception(ex)
            message = 'Error pushing item %s: %s' % (response.status_code, response.text)
            self._raise_publish_error(response.status_code, Exception(message), destination)

    def _copy_published_media_files(self, item, destination):
        """Copy the media files for the given item to the publish_items endpoint

        @param item: the item object
        @type item: dict
        @param assets_url: the url where the media can be uploaded
        @type assets_url: string
        """

        assets_url = self._get_assets_url(destination)

        if not (type(assets_url) == str and assets_url.strip()):
            return

        def parse_media(item):
            media = {}
            renditions = item.get('renditions', {})
            for _, rendition in renditions.items():
                rendition.pop('href', None)
                rendition.setdefault('mimetype', rendition.get('original', {}).get('mimetype', item.get('mimetype')))
                media[rendition['media']] = rendition
            for attachment in item.get('attachments', []):
                media.update({attachment['media']: {
                    'mimetype': attachment['mimetype'],
                    'resource': 'attachments',
                }})
            return media

        media = {}
        media.update(parse_media(item))

        for assoc in item.get('associations', {}).values():
            if assoc is None:
                continue
            media.update(parse_media(assoc))
            for assoc2 in assoc.get('associations', {}).values():
                if assoc2 is None:
                    continue
                media.update(parse_media(assoc2))

        for media_id, rendition in media.items():
            if not self._media_exists(media_id, destination):
                binary = app.media.get(media_id, resource=rendition.get('resource', 'upload'))
                self._transmit_media(binary, destination, exists=False)

    def _transmit_media(self, media, destination, exists=None):
        if exists is None:
            exists = self._media_exists(media._id, destination)
        if exists:
            return
        mimetype = getattr(media, 'content_type', 'image/jpeg')
        data = {'media_id': str(media._id)}
        files = {'media': (str(media._id), media, mimetype)}
        s = requests.Session()
        assets_url = self._get_assets_url(destination)
        request = requests.Request('POST', assets_url)
        prepped = request.prepare()
        prepped.prepare_body(data, files)
        headers = self._get_headers(prepped.body, destination, prepped.headers)
        prepped.prepare_headers(headers)
        response = s.send(prepped)
        if response.status_code not in (200, 201):
            self._raise_publish_error(
                response.status_code,
                Exception('Error pushing media file %s: %s %s' % (
                    str(media._id), response.status_code, response.text
                )),
                destination
            )

    def _media_exists(self, media_id, destination):
        """Returns true if the media with the given id exists at the service identified by assets_url.

        Returns false otherwise. Raises Exception if the error code was not 200 or 404

        @param media_id: the media identifier
        @type media_id: string
        @param assets_url: the url of the assest service
        @type assets_url: string
        @return: bool
        """
        assets_url = self._get_assets_url(destination, media_id)
        response = requests.get(assets_url)
        if response.status_code not in (requests.codes.ok, requests.codes.not_found):  # @UndefinedVariable
            self._raise_publish_error(
                response.status_code,
                Exception('Error querying the assets service %s' % assets_url),
                destination
            )
        return response.status_code == requests.codes.ok  # @UndefinedVariable

    def _get_headers(self, data, destination, current_headers):
        secret_token = self._get_secret_token(destination)
        if not secret_token:
            return current_headers
        data_hash = self._get_data_hash(data, secret_token)
        headers = current_headers.copy()
        headers[self.hash_header] = data_hash
        return headers

    def _get_data_hash(self, data, secret_token):
        if isinstance(data, str):
            encoded_data = bytes(data, 'utf-8')
        else:
            encoded_data = data
        mac = hmac.new(str.encode(secret_token), msg=encoded_data, digestmod='sha1')
        return 'sha1=' + str(mac.hexdigest())

    def _get_secret_token(self, destination):
        return destination.get('config', {}).get('secret_token', None)

    def _get_assets_url(self, destination, media_id=None):
        url = destination.get('config', {}).get('assets_url', None)
        if media_id is not None:
            return '/'.join([url, str(media_id)])
        return url

    def _get_resource_url(self, destination):
        return destination.get('config', {}).get('resource_url')

    def _raise_publish_error(self, status_code, e, destination=None):
        if status_code >= 400 and status_code < 500:
            raise PublishHTTPPushClientError.httpPushError(e, destination)
        elif status_code >= 500 and status_code < 600:
            raise PublishHTTPPushServerError.httpPushError(e, destination)
        else:
            raise PublishHTTPPushError.httpPushError(e, destination)


register_transmitter('http_push', HTTPPushService(), errors)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
import json
from eve.utils import ParsedRequest, config


class DeleteArchivedDocumentCommand(superdesk.Command):
    """
    Deletes a Text Archive document(s) from both Mongodb and ElasticSearch.
    It deletes digital package and the story given by id.
    It accepts one or more ids separated by space.

    manage.py app:deleteArchivedDocument 588c1df11d41c80928015601 588c1b901d41c805dce70df0
    """

    capture_all_args = True

    def can_delete_items(self, items):
        """Checks if the given items are deletable"""

        archived_service = superdesk.get_resource_service('archived')
        can_delete = True
        messages = []

        for item in items:
            try:
                archived_service.validate_delete_action(item, True)
            except Exception as ex:
                can_delete = False
                messages.append('-' * 45)
                messages.append('Cannot delete {} as {}'.format(item['item_id'], str(ex)))

        if not can_delete:
            [print(m) for m in messages]

        return can_delete

    def get_archived_items(self, ids):
        """Returns the items with the given list of ids"""

        query = {
            'query': {
                'filtered': {
                    'filter': {
                        'and': [
                            {'terms': {'_id': ids}}
                        ]
                    }
                }
            }
        }

        request = ParsedRequest()
        request.args = {'source': json.dumps(query)}
        return list(superdesk.get_resource_service('archived').get(req=request, lookup=None))

    def delete(self, items):
        """Deletes the given items and any digital package of them"""

        archived_service = superdesk.get_resource_service('archived')
        for item in items:
            articles_to_kill = archived_service.find_articles_to_kill({'_id': item[config.ID_FIELD]}, False)

            if not articles_to_kill:
                continue

            for article in articles_to_kill:
                archived_service.command_delete({'_id': article[config.ID_FIELD]})
                print("Deleted item {} ".format(article[config.ID_FIELD]))

    def run(self, ids):
        if ids and len(ids) > 0:
            items = self.get_archived_items(ids)

            if not items:
                print("No archived story found with given ids(s)!")
                return

            if self.can_delete_items(items):
                self.delete(items)
                print("Delete has been completed")

        else:
            print("Please provide at least one id!")
            return


superdesk.command('app:deleteArchivedDocument', DeleteArchivedDocumentCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
from datetime import timedelta

from flask import current_app as app
from superdesk import get_resource_service
from superdesk.celery_task_utils import get_lock_id
from superdesk.lock import lock, unlock
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)


class RemoveExpiredItems(superdesk.Command):
    """Remove expired items from the content_api items collection"""

    log_msg = ''
    expiry_days = 0  # by default this should not run

    option_list = [
        superdesk.Option('--expiry', '-m', dest='expiry_days', required=False)
    ]

    def run(self, expiry_days=None):
        if expiry_days:
            self.expiry_days = int(expiry_days)
        elif app.settings.get('CONTENT_API_EXPIRY_DAYS'):
            self.expiry_days = app.settings['CONTENT_API_EXPIRY_DAYS']

        if self.expiry_days == 0:
            logger.info('Expiry days is set to 0, therefor no items will be removed.')
            return

        now = utcnow()
        self.log_msg = 'Expiry Time: {}'.format(now)
        logger.info('{} Starting to remove expired content_api items.'.format(self.log_msg))

        lock_name = get_lock_id('content_api', 'remove_expired')
        if not lock(lock_name, expire=600):
            logger.info('{} Remove expired content_api items task is already running'.format(self.log_msg))
            return

        try:
            num_items_removed = self._remove_expired_items(now, self.expiry_days)
        finally:
            unlock(lock_name)

        if num_items_removed == 0:
            logger.info('{} Completed but no items were removed'.format(self.log_msg))
        else:
            logger.info('{} Completed removing {} expired content_api items'.format(self.log_msg, num_items_removed))

    def _remove_expired_items(self, expiry_datetime, expiry_days):
        """Remove expired items from content_api items

        :param datetime expiry_datetime: the datetime items are to expire
        :param int expiry_days: The number of days an item will be active
        """
        logger.info('{} Starting to remove expired items.'.format(self.log_msg))
        items_service = get_resource_service('items')

        num_items_removed = 0
        for expired_items in items_service.get_expired_items(
                expiry_datetime=expiry_datetime,
                expiry_days=expiry_days,
                include_children=False
        ):
            items_to_remove = set()

            log_msg_format = "{{'_id': {_id}, '_updated': {_updated}, 'expired_on': {expiry}}}"
            for item in expired_items:
                item.setdefault('expiry', item['_updated'] + timedelta(days=self.expiry_days))
                expiry_msg = log_msg_format.format(**item)
                logger.info('{} Processing expired item. {}'.format(self.log_msg, expiry_msg))

                for child in self._get_expired_chain(items_service, item, expiry_datetime):
                    items_to_remove.add(child['_id'])

            if items_to_remove:
                logger.info('{} Deleting items.: {}'.format(self.log_msg, items_to_remove))
                num_items_removed += len(items_to_remove)
                items_service.delete_action(lookup={'_id': {'$in': list(items_to_remove)}})

        logger.info('{} Finished removing expired items from the content_api'.format(self.log_msg))
        return num_items_removed

    def _get_expired_chain(self, service, parent, expiry_datetime):
        """Gets the list of expired items if they are all expired, otherwise it returns an empty list

        :param: service: The content_api items service
        :param: dict parent: The item to be removed
        :param: datetime expiry_datetime: The date and time items should be expired
        :return list: List of expired items if they're all expired, else an empty list
        """

        items = self._get_children(service, parent)
        items.append(parent)

        for item in items:
            if not self._has_expired(item, expiry_datetime):
                return []
        return items

    def _has_expired(self, item, expiry_datetime):
        """Checks if the item has expired

        :param dict item: The item to check
        :param datetime expiry_datetime: The date and time items should be expired
        :return bool: True if the item has expired, otherwise False
        """
        item.setdefault('expiry', item['_updated'] + timedelta(days=self.expiry_days))
        return item.get('expiry') <= expiry_datetime

    def _get_children(self, service, item):
        """Get the list of children to the root item using the ancestors dictionary key

        :param service: The content_api items service
        :param dict item: The root item to get the children of
        :return list: The list of children for this root item
        """
        return list(service.find({'ancestors': item['_id']}))


superdesk.command('content_api:remove_expired', RemoveExpiredItems())



import uuid
from lxml import etree

from .base import BasePackageElement


class Styles(BasePackageElement):
    """
    As Adobe's idml specification explains:
      ``
      The Styles.xml file contains all of the paragraph, character, object, cell, table,
      and table of contents (TOC) styles used in the document.
      ``
    """

    XML_DECLARATION = '<?xml version="1.0" encoding="utf-8" standalone="yes"?>'
    # http://wwwimages.adobe.com/content/dam/acom/en/devnet/indesign/sdk/cs6/idml/idml-specification.pdf
    # page 430
    PARAGRAPHSTYLE_DEFAULTS = {
        'Imported': 'false',
        'SplitDocument': 'false',
        'EmitCss': 'true',
        'StyleUniqueId': '$ID/',
        'IncludeClass': 'true',
        'EmptyNestedStyles': 'true',
        'EmptyLineStyles': 'true',
        'EmptyGrepStyles': 'true',
        'FillColor': 'Color/Black',
        'FontStyle': 'Regular',
        'PointSize': '12',
        'HorizontalScale': '100',
        'KerningMethod': '$ID/Metrics',
        'Ligatures': 'true',
        'PageNumberType': 'AutoPageNumber',
        'StrokeWeight': '1',
        'Tracking': '0',
        'Composer': 'HL Composer',
        'DropCapCharacters': '0',
        'DropCapLines': '0',
        'BaselineShift': '0',
        'Capitalization': 'Normal',
        'StrokeColor': 'Swatch/None',
        'HyphenateLadderLimit': '3',
        'VerticalScale': '100',
        'LeftIndent': '0',
        'RightIndent': '0',
        'FirstLineIndent': '0',
        'AutoLeading': '120',
        'AppliedLanguage': '$ID/English: USA',
        'Hyphenation': 'true',
        'HyphenateAfterFirst': '2',
        'HyphenateBeforeLast': '2',
        'HyphenateCapitalizedWords': 'true',
        'HyphenateWordsLongerThan': '5',
        'NoBreak': 'false',
        'HyphenationZone': '36',
        'SpaceBefore': '0',
        'SpaceAfter': '0',
        'Underline': 'false',
        'OTFFigureStyle': 'Default',
        'DesiredWordSpacing': '100',
        'MaximumWordSpacing': '133',
        'MinimumWordSpacing': '80',
        'DesiredLetterSpacing': '0',
        'MaximumLetterSpacing': '0',
        'MinimumLetterSpacing': '0',
        'DesiredGlyphScaling': '100',
        'MaximumGlyphScaling': '100',
        'MinimumGlyphScaling': '100',
        'StartParagraph': 'Anywhere',
        'KeepAllLinesTogether': 'false',
        'KeepWithNext': '0',
        'KeepFirstLines': '2',
        'KeepLastLines': '2',
        'Position': 'Normal',
        'StrikeThru': 'false',
        'CharacterAlignment': 'AlignEmCenter',
        'KeepLinesTogether': 'false',
        'StrokeTint': '-1',
        'FillTint': '-1',
        'OverprintStroke': 'false',
        'OverprintFill': 'false',
        'GradientStrokeAngle': '0',
        'GradientFillAngle': '0',
        'GradientStrokeLength': '-1',
        'GradientFillLength': '-1',
        'GradientStrokeStart': '0 0',
        'GradientFillStart': '0 0',
        'Skew': '0',
        'RuleAboveLineWeight': '1',
        'RuleAboveTint': '-1',
        'RuleAboveOffset': '0',
        'RuleAboveLeftIndent': '0',
        'RuleAboveRightIndent': '0',
        'RuleAboveWidth': 'ColumnWidth',
        'RuleBelowLineWeight': '1',
        'RuleBelowTint': '-1',
        'RuleBelowOffset': '0',
        'RuleBelowLeftIndent': '0',
        'RuleBelowRightIndent': '0',
        'RuleBelowWidth': 'ColumnWidth',
        'RuleAboveOverprint': 'false',
        'RuleBelowOverprint': 'false',
        'RuleAbove': 'false',
        'RuleBelow': 'false',
        'LastLineIndent': '0',
        'HyphenateLastWord': 'true',
        'ParagraphBreakType': 'Anywhere',
        'SingleWordJustification': 'FullyJustified',
        'OTFOrdinal': 'false',
        'OTFFraction': 'false',
        'OTFDiscretionaryLigature': 'false',
        'OTFTitling': 'false',
        'RuleAboveGapTint': '-1',
        'RuleAboveGapOverprint': 'false',
        'RuleBelowGapTint': '-1',
        'RuleBelowGapOverprint': 'false',
        'Justification': 'LeftAlign',
        'DropcapDetail': '1',
        'PositionalForm': 'None',
        'OTFMark': 'true',
        'HyphenWeight': '5',
        'OTFLocale': 'true',
        'HyphenateAcrossColumns': 'true',
        'KeepRuleAboveInFrame': 'false',
        'IgnoreEdgeAlignment': 'false',
        'OTFSlashedZero': 'false',
        'OTFStylisticSets': '0',
        'OTFHistorical': 'false',
        'OTFContextualAlternate': 'true',
        'UnderlineGapOverprint': 'false',
        'UnderlineGapTint': '-1',
        'UnderlineOffset': '-9999',
        'UnderlineOverprint': 'false',
        'UnderlineTint': '-1',
        'UnderlineWeight': '-9999',
        'StrikeThroughGapOverprint': 'false',
        'StrikeThroughGapTint': '-1',
        'StrikeThroughOffset': '-9999',
        'StrikeThroughOverprint': 'false',
        'StrikeThroughTint': '-1',
        'StrikeThroughWeight': '-9999',
        'MiterLimit': '4',
        'StrokeAlignment': 'OutsideAlignment',
        'EndJoin': 'MiterEndJoin',
        'SpanColumnType': 'SingleColumn',
        'SplitColumnInsideGutter': '6',
        'SplitColumnOutsideGutter': '0',
        'KeepWithPrevious': 'false',
        'SpanColumnMinSpaceBefore': '0',
        'SpanColumnMinSpaceAfter': '0',
        'OTFSwash': 'false',
        'ParagraphShadingTint': '-1',
        'ParagraphShadingOverprint': 'false',
        'ParagraphShadingWidth': 'ColumnWidth',
        'ParagraphShadingOn': 'false',
        'ParagraphShadingClipToFrame': 'false',
        'ParagraphShadingSuppressPrinting': 'false',
        'ParagraphShadingLeftOffset': '0',
        'ParagraphShadingRightOffset': '0',
        'ParagraphShadingTopOffset': '0',
        'ParagraphShadingBottomOffset': '0',
        'ParagraphShadingTopOrigin': 'AscentTopOrigin',
        'ParagraphShadingBottomOrigin': 'DescentBottomOrigin',
        'ParagraphBorderTint': '-1',
        'ParagraphBorderOverprint': 'false',
        'ParagraphBorderOn': 'false',
        'ParagraphBorderGapTint': '-1',
        'ParagraphBorderGapOverprint': 'false',
        'Tsume': '0',
        'LeadingAki': '-1',
        'TrailingAki': '-1',
        'KinsokuType': 'KinsokuPushInFirst',
        'KinsokuHangType': 'None',
        'BunriKinshi': 'true',
        'RubyOpenTypePro': 'true',
        'RubyFontSize': '-1',
        'RubyAlignment': 'RubyJIS',
        'RubyType': 'PerCharacterRuby',
        'RubyParentSpacing': 'RubyParent121Aki',
        'RubyXScale': '100',
        'RubyYScale': '100',
        'RubyXOffset': '0',
        'RubyYOffset': '0',
        'RubyPosition': 'AboveRight',
        'RubyAutoAlign': 'true',
        'RubyParentOverhangAmount': 'RubyOverhangOneRuby',
        'RubyOverhang': 'false',
        'RubyAutoScaling': 'false',
        'RubyParentScalingPercent': '66',
        'RubyTint': '-1',
        'RubyOverprintFill': 'Auto',
        'RubyStrokeTint': '-1',
        'RubyOverprintStroke': 'Auto',
        'RubyWeight': '-1',
        'KentenKind': 'None',
        'KentenFontSize': '-1',
        'KentenXScale': '100',
        'KentenYScale': '100',
        'KentenPlacement': '0',
        'KentenAlignment': 'AlignKentenCenter',
        'KentenPosition': 'AboveRight',
        'KentenCustomCharacter': '',
        'KentenCharacterSet': 'CharacterInput',
        'KentenTint': '-1',
        'KentenOverprintFill': 'Auto',
        'KentenStrokeTint': '-1',
        'KentenOverprintStroke': 'Auto',
        'KentenWeight': '-1',
        'Tatechuyoko': 'false',
        'TatechuyokoXOffset': '0',
        'TatechuyokoYOffset': '0',
        'AutoTcy': '0',
        'AutoTcyIncludeRoman': 'false',
        'Jidori': '0',
        'GridGyoudori': '0',
        'GridAlignFirstLineOnly': 'false',
        'GridAlignment': 'None',
        'CharacterRotation': '0',
        'RotateSingleByteCharacters': 'false',
        'Rensuuji': 'true',
        'ShataiMagnification': '0',
        'ShataiDegreeAngle': '4500',
        'ShataiAdjustTsume': 'true',
        'ShataiAdjustRotation': 'false',
        'Warichu': 'false',
        'WarichuLines': '2',
        'WarichuSize': '50',
        'WarichuLineSpacing': '0',
        'WarichuAlignment': 'Auto',
        'WarichuCharsBeforeBreak': '2',
        'WarichuCharsAfterBreak': '2',
        'OTFHVKana': 'false',
        'OTFProportionalMetrics': 'false',
        'OTFRomanItalics': 'false',
        'LeadingModel': 'LeadingModelAkiBelow',
        'ScaleAffectsLineHeight': 'false',
        'ParagraphGyoudori': 'false',
        'CjkGridTracking': 'false',
        'GlyphForm': 'None',
        'RubyAutoTcyDigits': '0',
        'RubyAutoTcyIncludeRoman': 'false',
        'RubyAutoTcyAutoScale': 'true',
        'TreatIdeographicSpaceAsSpace': 'true',
        'AllowArbitraryHyphenation': 'false',
        'BulletsAndNumberingListType': 'NoList',
        'NumberingStartAt': '1',
        'NumberingLevel': '1',
        'NumberingContinue': 'true',
        'NumberingApplyRestartPolicy': 'true',
        'BulletsAlignment': 'LeftAlign',
        'NumberingAlignment': 'LeftAlign',
        'NumberingExpression': '^#.^t',
        'BulletsTextAfter': '^t',
        'ParagraphBorderLeftOffset': '0',
        'ParagraphBorderRightOffset': '0',
        'ParagraphBorderTopOffset': '0',
        'ParagraphBorderBottomOffset': '0',
        'ParagraphBorderStrokeEndJoin': 'MiterEndJoin',
        'ParagraphBorderTopLeftCornerOption': 'None',
        'ParagraphBorderTopRightCornerOption': 'None',
        'ParagraphBorderBottomLeftCornerOption': 'None',
        'ParagraphBorderBottomRightCornerOption': 'None',
        'ParagraphBorderTopLeftCornerRadius': '1',
        'ParagraphBorderTopRightCornerRadius': '1',
        'ParagraphBorderBottomLeftCornerRadius': '1',
        'ParagraphBorderBottomRightCornerRadius': '1',
        'ParagraphShadingTopLeftCornerOption': 'None',
        'ParagraphShadingTopRightCornerOption': 'None',
        'ParagraphShadingBottomLeftCornerOption': 'None',
        'ParagraphShadingBottomRightCornerOption': 'None',
        'ParagraphShadingTopLeftCornerRadius': '1',
        'ParagraphShadingTopRightCornerRadius': '1',
        'ParagraphShadingBottomLeftCornerRadius': '1',
        'ParagraphShadingBottomRightCornerRadius': '1',
        'ParagraphBorderStrokeEndCap': 'ButtEndCap',
        'ParagraphBorderWidth': 'ColumnWidth',
        'ParagraphBorderTopOrigin': 'AscentTopOrigin',
        'ParagraphBorderBottomOrigin': 'DescentBottomOrigin',
        'ParagraphBorderTopLineWeight': '1',
        'ParagraphBorderBottomLineWeight': '1',
        'ParagraphBorderLeftLineWeight': '1',
        'ParagraphBorderRightLineWeight': '1',
        'ParagraphBorderDisplayIfSplits': 'false',
        'MergeConsecutiveParaBorders': 'false',
        'ProviderHyphenationStyle': 'HyphAll',
        'DigitsType': 'DefaultDigits',
        'Kashidas': 'DefaultKashidas',
        'DiacriticPosition': 'OpentypePositionFromBaseline',
        'CharacterDirection': 'DefaultDirection',
        'ParagraphDirection': 'LeftToRightDirection',
        'ParagraphJustification': 'DefaultJustification',
        'ParagraphKashidaWidth': '2',
        'XOffsetDiacritic': '0',
        'YOffsetDiacritic': '0',
        'OTFOverlapSwash': 'false',
        'OTFStylisticAlternate': 'false',
        'OTFJustificationAlternate': 'false',
        'OTFStretchedAlternate': 'false',
        'KeyboardDirection': 'DefaultDirection',
    }

    @property
    def filename(self):
        """
        Filename inside IDML package.
        Used as a filename for a file inside zip container.
        :return str: filename
        """
        return 'Resources/Styles.xml'

    def _build_etree(self):
        self._etree = etree.Element(
            etree.QName(self.XMLNS_IDPKG, 'Styles'),
            nsmap={'idPkg': self.XMLNS_IDPKG}
        )
        self._etree.set('DOMVersion', self.DOM_VERSION)
        self._add_paragraph_style_group_paragraphs()
        self._add_paragraph_style_group_headings()
        self._add_paragraph_style_group_lists()
        self._add_table_style_group()
        self._add_character_style_group()
        self._add_paragraph_style_group_custom()

    def _add_paragraph_style_group_paragraphs(self):
        """
        Add paragraphs style group
        """
        # RootParagraphStyleGroup
        self._rootparagraphstylegroup = etree.SubElement(
            self._etree,
            'RootParagraphStyleGroup',
            attrib={
                'Self': 'root_paragraph_style_group',
            }
        )
        # ParagraphStyleGroup $ID/Paragraphs
        paragraphstylegroup = etree.SubElement(
            self._rootparagraphstylegroup,
            'ParagraphStyleGroup',
            attrib={
                'Self': 'ParagraphStyleGroup/$ID/Paragraphs',
                'Name': '$ID/Paragraphs'
            }
        )
        # ParagraphStyle.Paragraphs:NormalParagraph
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Paragraphs%3aNormalParagraph',
                'Name': 'Paragraphs:NormalParagraph',
                'NextStyle': 'ParagraphStyle/Paragraphs%3aNormalParagraph',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4())
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = '$ID/[No paragraph style]'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '12'
        # ParagraphStyle.Paragraphs:Blockquote
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Paragraphs%3aBlockquote',
                'Name': 'Paragraphs:Blockquote',
                'NextStyle': 'ParagraphStyle/Paragraphs%3aBlockquote',
                'KeyboardShortcut': '0 0',
                'FontStyle': 'Italic',
                'LeftIndent': '30',
                'RightIndent': '30',
                'ParagraphBorderOn': 'true',
                'ParagraphBorderLeftOffset': '-20',
                'ParagraphBorderTopLineWeight': '0',
                'ParagraphBorderBottomLineWeight': '0',
                'ParagraphBorderLeftLineWeight': '4',
                'ParagraphBorderRightLineWeight': '0',
                'StyleUniqueId': str(uuid.uuid4())
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = '$ID/[No paragraph style]'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '12'
        # ParagraphStyle.Paragraphs:Preformatted
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Paragraphs%3aPreformatted',
                'Name': 'Paragraphs:Preformatted',
                'NextStyle': 'ParagraphStyle/Paragraphs%3aPreformatted',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4())
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = '$ID/[No paragraph style]'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '12'
        etree.SubElement(
            properties, 'AppliedFont', attrib={'type': 'string'}
        ).text = 'Courier'
        # ParagraphStyle.Paragraphs:Preformatted
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Paragraphs%3aNormalParagraph',
                'Name': 'Paragraphs:NormalParagraph',
                'NextStyle': 'ParagraphStyle/Paragraphs%3aNormalParagraph',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4())
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = '$ID/[No paragraph style]'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '12'

    def _add_paragraph_style_group_headings(self):
        """
        Add headings style group
        """
        # ParagraphStyleGroup
        paragraphstylegroup = etree.SubElement(
            self._rootparagraphstylegroup,
            'ParagraphStyleGroup',
            attrib={
                'Self': 'ParagraphStyleGroup/$ID/Headings',
                'Name': '$ID/Headings'
            }
        )
        # ParagraphStyle.Headings:Heading1
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Headings%3aHeading1',
                'Name': 'Headings:Heading1',
                'NextStyle': 'ParagraphStyle/Headings%3aHeading1',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
                'PointSize': '40',
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = 'ParagraphStyle/$ID/NormalParagraphStyle'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'ParagraphShadingColor', attrib={'type': 'object'}
        ).text = 'Color/Black'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '40'
        # ParagraphStyle.Headings:Heading2
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Headings%3aHeading2',
                'Name': 'Headings:Heading2',
                'NextStyle': 'ParagraphStyle/Headings%3aHeading2',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
                'PointSize': '30',
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = 'ParagraphStyle/$ID/NormalParagraphStyle'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'ParagraphShadingColor', attrib={'type': 'object'}
        ).text = 'Color/Black'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '30'
        # ParagraphStyle.Headings:Heading3
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Headings%3aHeading3',
                'Name': 'Headings:Heading3',
                'NextStyle': 'ParagraphStyle/Headings%3aHeading3',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
                'PointSize': '20',
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = 'ParagraphStyle/$ID/NormalParagraphStyle'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'ParagraphShadingColor', attrib={'type': 'object'}
        ).text = 'Color/Black'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '20'
        # ParagraphStyle.Headings:Heading4
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Headings%3aHeading4',
                'Name': 'Headings:Heading4',
                'NextStyle': 'ParagraphStyle/Headings%3aHeading4',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
                'PointSize': '14',
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = 'ParagraphStyle/$ID/NormalParagraphStyle'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'ParagraphShadingColor', attrib={'type': 'object'}
        ).text = 'Color/Black'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '14'
        # ParagraphStyle.Headings:Heading5
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Headings%3aHeading5',
                'Name': 'Headings:Heading5',
                'NextStyle': 'ParagraphStyle/Headings%3aHeading5',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
                'PointSize': '11',
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = 'ParagraphStyle/$ID/NormalParagraphStyle'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'ParagraphShadingColor', attrib={'type': 'object'}
        ).text = 'Color/Black'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '11'
        # ParagraphStyle.Headings:Heading6
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Headings%3aHeading6',
                'Name': 'Headings:Heading6',
                'NextStyle': 'ParagraphStyle/Headings%3aHeading6',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
                'PointSize': '9',
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = 'ParagraphStyle/$ID/NormalParagraphStyle'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'ParagraphShadingColor', attrib={'type': 'object'}
        ).text = 'Color/Black'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '9'

    def _add_paragraph_style_group_lists(self):
        """
        Add lists style group
        """
        # ParagraphStyleGroup
        paragraphstylegroup = etree.SubElement(
            self._rootparagraphstylegroup,
            'ParagraphStyleGroup',
            attrib={
                'Self': 'ParagraphStyleGroup/$ID/Lists',
                'Name': '$ID/Lists'
            }
        )
        # ParagraphStyle.Lists:OrderedList
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Lists%3aOrderedList',
                'Name': 'Lists:OrderedList',
                'NextStyle': 'ParagraphStyle/Lists%3aOrderedList',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = '$ID/[No paragraph style]'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '12'
        # ParagraphStyle.Lists:UnorderedList
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Lists%3aUnorderedList',
                'Name': 'Lists:UnorderedList',
                'NextStyle': 'ParagraphStyle/Lists%3aUnorderedList',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = '$ID/[No paragraph style]'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '12'

    def _add_table_style_group(self):
        """
        Add table style group
        """
        # RootTableStyleGroup
        self._roottablestylegroup = etree.SubElement(
            self._etree,
            'RootTableStyleGroup',
            attrib={
                'Self': 'root_table_style_group',
            }
        )
        # TableStyle
        tablestyle = etree.SubElement(
            self._roottablestylegroup,
            'TableStyle',
            attrib={
                'Self': 'TableStyle/$ID/NormalTable',
                'Name': '$ID/NormalTable',
                'Imported': 'false',
                'NextStyle': 'TableStyle/$ID/NormalTable',
                'KeyboardShortcut': '0 0'
            }
        )
        # Properties
        properties = etree.SubElement(tablestyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = '$ID/[No table style]'

    def _add_character_style_group(self):
        """
        Add characters style group
        """
        # RootCharacterStyleGroup
        self._rootcharacterstylegroup = etree.SubElement(
            self._etree,
            'RootCharacterStyleGroup',
            attrib={
                'Self': 'root_character_style_group',
            }
        )
        # CharacterStyle No character style
        etree.SubElement(
            self._rootcharacterstylegroup,
            'CharacterStyle',
            attrib={
                'Self': 'CharacterStyle/$ID/[No character style]',
                'Name': '$ID/[No character style]'
            }
        )
        # CharacterStyle.Hyperlink
        characterstyle = etree.SubElement(
            self._rootcharacterstylegroup,
            'CharacterStyle',
            attrib={
                'Self': 'CharacterStyle/$ID/Hyperlink',
                'Name': '$ID/Hyperlink',
                'FillColor': 'Color/Hyperlink',
                'Underline': 'true',
            }
        )
        # Properties
        properties = etree.SubElement(characterstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = '$ID/[No character style]'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'

    def _add_paragraph_style_group_custom(self):
        """
        Add custom style group
        """
        # ParagraphStyleGroup
        paragraphstylegroup = etree.SubElement(
            self._rootparagraphstylegroup,
            'ParagraphStyleGroup',
            attrib={
                'Self': 'ParagraphStyleGroup/$ID/Custom',
                'Name': '$ID/Custom'
            }
        )

        # ParagraphStyle.Custom:Headline
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Custom%3aHeadline',
                'Name': 'Custom:Headline',
                'NextStyle': 'ParagraphStyle/Custom%3aHeadline',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
                'FontStyle': 'Bold',
                'PointSize': '48',
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = 'ParagraphStyle/$ID/NormalParagraphStyle'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'ParagraphShadingColor', attrib={'type': 'object'}
        ).text = 'Color/Black'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '48'
        # ParagraphStyle.Custom:Byline
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            'ParagraphStyle',
            attrib={
                'Self': 'ParagraphStyle/Custom%3aByline',
                'Name': 'Custom:Byline',
                'NextStyle': 'ParagraphStyle/Custom%3aByline',
                'KeyboardShortcut': '0 0',
                'StyleUniqueId': str(uuid.uuid4()),
                'FontStyle': 'Bold',
                'PointSize': '20',
            }
        )
        properties = etree.SubElement(paragraphstyle, 'Properties')
        etree.SubElement(
            properties, 'BasedOn', attrib={'type': 'string'}
        ).text = 'ParagraphStyle/$ID/NormalParagraphStyle'
        etree.SubElement(
            properties, 'PreviewColor', attrib={'type': 'enumeration'}
        ).text = 'Nothing'
        etree.SubElement(
            properties, 'ParagraphShadingColor', attrib={'type': 'object'}
        ).text = 'Color/Black'
        etree.SubElement(
            properties, 'Leading', attrib={'type': 'unit'}
        ).text = '20'



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import unittest

from datetime import timedelta
from superdesk.utc import utcnow
from superdesk.etree import etree
from superdesk.text_utils import get_word_count, get_char_count
from superdesk.io.registry import registered_feed_parsers
from superdesk.io.feed_parsers.newsml_1_2 import NewsMLOneFeedParser
from superdesk.io.feed_parsers.newsml_2_0 import NewsMLTwoFeedParser
from superdesk.io.feed_parsers.nitf import NITFFeedParser
from superdesk.io.feeding_services.file_service import FileFeedingService


def get_etree(filename):
    dirname = os.path.dirname(os.path.realpath(__file__))
    with open(os.path.join(dirname, 'fixtures', filename)) as f:
        return etree.fromstring(f.read().encode('utf-8'))


class UtilsTest(unittest.TestCase):

    def test_get_word_count(self):
        self.assertEqual(2, get_word_count('plain text'), 'plain text')
        self.assertEqual(2, get_word_count('<p> html text </p>'), 'paragraph')

        self.assertEqual(22, get_word_count(
            '<doc><p xml:lang="en-US">The weather was superb today in Norfolk, Virginia. Made me want to take\n'
            'out my boat, manufactured by the <org value="acm" idsrc="iptc.org">Acme Boat Company</org>.</p></doc>'))

    def test_get_char_count(self):
        self.assertEqual(10, get_char_count('plain text'))
        self.assertEqual(11, get_char_count('<p> html text </p>'))

        self.assertEqual(123, get_char_count(
            '<doc><p xml:lang="en-US">The weather was superb today in Norfolk, Virginia. Made me want to take\n'
            'out my boat, manufactured by the <org value="acm" idsrc="iptc.org">Acme Boat Company</org>.</p></doc>'))

    def test_get_xml_parser_newsmlg2(self):
        etree = get_etree('snep.xml')
        self.assertIsInstance(FileFeedingService().get_feed_parser({'feed_parser': 'newsml2'}, etree),
                              NewsMLTwoFeedParser)

    def test_get_xml_parser_nitf(self):
        etree = get_etree('nitf-fishing.xml')
        self.assertIsInstance(FileFeedingService().get_feed_parser({'feed_parser': 'nitf'}, etree),
                              NITFFeedParser)

    def test_get_xml_parser_newsml12(self):
        etree = get_etree('afp.xml')
        self.assertIsInstance(FileFeedingService().get_feed_parser({'feed_parser': 'newsml12'}, etree),
                              NewsMLOneFeedParser)

    def test_is_old_content(self):
        service = FileFeedingService()
        self.assertFalse(service.is_old_content(utcnow()))
        self.assertTrue(service.is_old_content(utcnow() - timedelta(minutes=11)))


class ItemTest(unittest.TestCase):

    def setUpFixture(self, filename):
        self.tree = get_etree(filename)
        provider = {'name': 'Test'}

        for parser in registered_feed_parsers.values():
            if parser.can_parse(self.tree):
                self.item = parser.parse(self.tree, provider)[0]


class TextParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture('text.xml')

    def test_instance(self):
        self.assertTrue(self.item)

    def test_parse_id(self):
        self.assertEqual("tag:reuters.com,0000:newsml_L4N0BT5PJ:263518268", self.item.get('guid'))
        self.assertEqual('263518268', self.item.get('version'))
        self.assertEqual(3, self.item.get('priority'))

    def test_parse_item_meta(self):
        self.assertEqual("text", self.item.get('type'))
        self.assertEqual("2013-03-01T15:09:04", self.item.get('versioncreated').isoformat())
        self.assertEqual("2013-03-01T15:09:04", self.item.get('firstcreated').isoformat())
        self.assertEqual("Editorial Note", self.item.get('ednote'))

    def test_parse_content_meta(self):
        self.assertEqual(3, self.item.get('urgency'))
        self.assertEqual("SOCCER-ENGLAND/CHELSEA-BENITEZ", self.item["slugline"])
        self.assertEqual("Soccer-Smiling Benitez pleads for support "
                         "after midweek outburst against opponent", self.item["headline"])
        self.assertNotIn('description_text', self.item)
        self.assertNotIn('archive_description', self.item)

    def test_content_set(self):
        self.assertEqual("<p>By Toby Davis</p>", self.item.get('body_html'))
        self.assertEqual(569, self.item.get('word_count'))
        self.assertIsInstance(self.item.get('body_html'), type(''))

    def test_language(self):
        self.assertEqual('en', self.item.get('language'))

    def test_subject(self):
        self.assertEqual(2, len(self.item.get('subject')))
        self.assertIn({'qcode': '15054000', 'name': 'soccer'}, self.item.get('subject'))

    def test_pubstatus(self):
        self.assertEqual('usable', self.item.get('pubstatus'))


class PictureParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture('picture.xml')

    def test_type(self):
        self.assertEqual('picture', self.item.get('type'))

    def test_content_set(self):
        self.assertEqual(3, len(self.item.get('renditions')))
        self.assertEqual(4, self.item.get('priority'))
        remote = self.item.get('renditions').get('baseImage')
        self.assertTrue(remote)
        self.assertEqual("tag:reuters.com,0000:binary_GM1E9341HD701-BASEIMAGE", remote.get('residRef'))
        self.assertEqual(772617, remote.get('sizeinbytes'))
        self.assertEqual("image/jpeg", remote.get('mimetype'))
        self.assertEqual("http://content.reuters.com/auth-server/content/tag:reuters.com,0000:newsml_GM1E9341HD701:360624134/tag:reuters.com,0000:binary_GM1E9341HD701-BASEIMAGE", remote.get('href'))  # noqa
        self.assertIn("A woman casts her ballot inside a polling station in Kenya's town of Gatundu "
                      "March 4, 2013. Polling stations opened up to Kenyans on Monday for a tense "
                      "presidential election that will test whether the east African nation can repair "
                      "its damaged reputation after the tribal blood-letting that followed a "
                      "2007 poll.    REUTERS/Marko Djurica (KENYA - Tags: POLITICS ELECTIONS)",
                      self.item['description_text'])
        self.assertIn("A woman casts her ballot inside a polling station in Kenya's town of Gatundu "
                      "March 4, 2013. Polling stations opened up to Kenyans on Monday for a tense "
                      "presidential election that will test whether the east African nation can repair "
                      "its damaged reputation after the tribal blood-letting that followed a "
                      "2007 poll.    REUTERS/Marko Djurica (KENYA - Tags: POLITICS ELECTIONS)",
                      self.item['archive_description'])

    def test_byline(self):
        self.assertEqual('MARKO DJURICA', self.item.get('byline'))

    def test_place(self):
        self.assertEqual(2, len(self.item.get('place')))
        self.assertIn({'name': 'NAIROBI'}, self.item['place'])
        self.assertIn({'name': 'Kenya'}, self.item['place'])


class SNEPParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture('snep.xml')

    def test_content_set(self):
        self.assertEqual(4, self.item.get('priority'))
        self.assertEqual(2, len(self.item.get('groups')))

        group = self.item.get('groups')[0]
        self.assertTrue(group)
        self.assertEqual("root", group.get('id'))
        self.assertEqual("grpRole:SNEP", group.get('role'))
        self.assertEqual(1, len(group.get('refs')))
        self.assertEqual("main", group.get('refs')[0].get('idRef'))

        group = self.item.get('groups')[1]
        self.assertEqual(10, len(group.get('refs')))
        self.assertEqual("main", group.get('id'))

        ref = group.get('refs')[0]
        self.assertTrue(ref)
        self.assertEqual("tag:reuters.com,0000:newsml_BRE9220HA:15", ref.get('residRef'))
        self.assertEqual("application/vnd.iptc.g2.packageitem+xml", ref.get('contentType'))
        self.assertEqual("icls:composite", ref.get('itemClass'))
        self.assertEqual("At least 15 killed on Kenya coast on election day", ref.get('headline'))
